#include "GeneratorThread.h"

GeneratorThread::GeneratorThread()
: ubudslLogDir( new QDir(QDir::tempPath() + "/ubudsl_log") )
{
	if ( !ubudslLogDir->exists() )
		ubudslLogDir->mkpath(ubudslLogDir->path());
	
	removeLogDirContent();
	
	logFile    = new QFile( ubudslLogDir->path() + "/ubudsl-ng.log" );
	kernelMsgs = new QFile( ubudslLogDir->path() + "/kernel.log" );
	m_gatheredInfo = "";
}

GeneratorThread::~GeneratorThread()
{
	removeLogDirContent();
	ubudslLogDir->rmpath(ubudslLogDir->path());
	
	logFile = 0;
	delete logFile;
	delete kernelMsgs;
	delete ubudslLogDir;
}

void GeneratorThread::removeLogDirContent()
{
	QStringList content = ubudslLogDir->entryList(QDir::NoDotAndDotDot|QDir::Files);
	
	foreach( QString file, content )
	{
		QFile::remove(ubudslLogDir->path() + "/" + file);
	}
}

void GeneratorThread::run()
{
	if (!logFile->open(QFile::WriteOnly|QFile::Truncate))
	{
		fprintf(stderr, "Fatal error: could not open %s in WriteOnly mode!\n", qPrintable(logFile->fileName()));
		emit writingFailed();
		return;
	}
	
	logFile->write(qPrintable(m_gatheredInfo));
	
	getSystemInfo();
	getUSBDevices();
	getIfaces();
	getDNSServers();
	getFirmwareInfo();
	getRoutingTable();
	getPermissions();
	
	logFile->close();
	
	getKernelMessages();
	
	QFile::copy( "/opt/ubudsl/daemon.log", ubudslLogDir->path() + "/daemon.log" );
	QFile::copy( "/opt/ubudsl/debug.txt", ubudslLogDir->path() + "/debug.txt" );
	
	QDir d("/opt/ubudsl/logs");
	QStringList logs = d.entryList(QDir::Files|QDir::NoSymLinks);
	
	foreach ( QString log, logs )
	{
		QFile::copy( d.absolutePath() + "/" + log, ubudslLogDir->path() + "/" + log );
	}
	
	QProcess::execute( "tar", QStringList() << "-C" << ubudslLogDir->path()
			                                  << "-cjf" <<  QDir::homePath() + "/ubudsl_log.tbz2"
			                                  << ubudslLogDir->entryList(QDir::Files|QDir::NoSymLinks) );
}

void GeneratorThread::getIfaces()
{
	QList<QNetworkInterface> ifaces = QNetworkInterface::allInterfaces();
	QListIterator<QNetworkInterface> ifacesIt(ifaces);
	
	while (ifacesIt.hasNext())
	{
		QNetworkInterface iface = ifacesIt.next();
		
		logFile->write("Interface ");
		logFile->write(qPrintable(iface.name()));
		logFile->write(":\n");
		
		logFile->write(qPrintable(QString("  Interface is up: ").leftJustified(30)));
		logFile->write(iface.flags().testFlag(QNetworkInterface::IsUp) ? "Yes\n" : "No\n");
		
		logFile->write(qPrintable(QString("  Interface is running: ").leftJustified(30)));
		logFile->write(iface.flags().testFlag(QNetworkInterface::IsRunning) ? "Yes\n" : "No\n");
		
		logFile->write(qPrintable(QString("  Interface is PointToPoint: ").leftJustified(30)));
		logFile->write(iface.flags().testFlag(QNetworkInterface::IsPointToPoint) ? "Yes\n" : "No\n");
		
		logFile->write(qPrintable(QString("  Interface is LoopBack: ").leftJustified(30)));
		logFile->write(iface.flags().testFlag(QNetworkInterface::IsLoopBack) ? "Yes\n" : "No\n");
		
		logFile->write(qPrintable(QString("  Interface can Multicast: ").leftJustified(30)));
		logFile->write(iface.flags().testFlag(QNetworkInterface::CanMulticast) ? "Yes\n" : "No\n");
		
		logFile->write(qPrintable(QString("  Interface can Broadcast: ").leftJustified(30)));
		logFile->write(iface.flags().testFlag(QNetworkInterface::CanBroadcast) ? "Yes\n" : "No\n");
		
		logFile->write(qPrintable(QString("  Hardware address: ").leftJustified(30)));
		logFile->write(qPrintable(iface.hardwareAddress()));
		logFile->write("\n");

		QList<QNetworkAddressEntry> addresses = iface.addressEntries();
		QListIterator<QNetworkAddressEntry> addressesIt(addresses);
		
		while (addressesIt.hasNext())
		{
			QNetworkAddressEntry address = addressesIt.next();
			
			logFile->write(qPrintable(QString("  Broadcast address: ").leftJustified(30)));
			logFile->write(qPrintable(address.broadcast().toString()));
			logFile->write("\n");
			
			logFile->write(qPrintable(QString("  IP address: ").leftJustified(30)));
			logFile->write(qPrintable(address.ip().toString()));
			logFile->write("\n");
			
			logFile->write(qPrintable(QString("  Netmask address: ").leftJustified(30)));
			logFile->write(qPrintable(address.netmask().toString()));
			logFile->write("\n");
		}
		
		logFile->write("\n\n");
	}
}

void GeneratorThread::getSystemInfo()
{
	logFile->write("System:\n");
	
	logFile->write(qPrintable(QString("  Distribution name: ").leftJustified(30)));
	logFile->write(qPrintable(UbuDSL::SystemInfo::getDistributionName()));
	logFile->write("\n");
	
	logFile->write(qPrintable(QString("  Distribution release: ").leftJustified(30)));
	logFile->write(qPrintable(UbuDSL::SystemInfo::getDistributionRelease()));
	logFile->write("\n");
	
	logFile->write(qPrintable(QString("  Kernel release: ").leftJustified(30)));
	logFile->write(qPrintable(UbuDSL::SystemInfo::getKernelRelease()));
	logFile->write("\n");
	
	logFile->write(qPrintable(QString("  Processor architecture: ").leftJustified(30)));
	logFile->write(qPrintable(UbuDSL::SystemInfo::getProcessorArchitecture()));
	logFile->write("\n");
	
	logFile->write(qPrintable(QString("  System architecture: ").leftJustified(30)));
	logFile->write(qPrintable(UbuDSL::SystemInfo::getArchitecture()));
	logFile->write("\n\n\n");
}

void GeneratorThread::getUSBDevices()
{
	logFile->write("USB devices:\n");

	QRegExp deviceRegExp("(\\s{2}Bus [0-9]{3} Device [0-9]{3}: )\\s*(ID .{4}:.{4}\\s+.{0,100})\\s*$");
	QProcess *lsusb = new QProcess;
	
	lsusb->start("lsusb");
	lsusb->waitForFinished();
	
	while (!lsusb->atEnd())
	{
		QString line = lsusb->readLine(150);
		if (line.indexOf("\n") > -1)
			line = line.remove(line.indexOf("\n", line.count()));
		
		line = line.prepend("  ");
		
		if (line.contains(deviceRegExp))
		{
			QString bus = deviceRegExp.cap(1);
			QString id  = deviceRegExp.cap(2);
			
			logFile->write(qPrintable(bus.leftJustified(30)));
			logFile->write(qPrintable(id));
		}
	}
	
	logFile->write("\n\n");
	lsusb = 0;
	delete lsusb;
}

void GeneratorThread::getRoutingTable()
{
	logFile->write("Routing table:\n");
	logFile->write("  ");
	logFile->write(qPrintable(QString("Destination").leftJustified(15)));
	logFile->write(qPrintable(QString("Gateway").leftJustified(15)));
	logFile->write(qPrintable(QString("Genmask").leftJustified(15)));
	logFile->write(qPrintable(QString("Flags").leftJustified(10)));
	logFile->write(qPrintable(QString("Metric").leftJustified(10)));
	logFile->write(qPrintable(QString("Ref").leftJustified(10)));
	logFile->write(qPrintable(QString("Use").leftJustified(10)));
	logFile->write(qPrintable(QString("Iface\n")));

	QString addr = "([0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3})";
	QString num  = "([0-9]{1,4})";
	QRegExp addrRegExp(addr+"\\s+"+addr+"\\s+"+addr+"\\s+([A-Z]{1,2})\\s+"+num+"\\s+"+num+"\\s+"+num+"\\s+([a-z]{2,4}[0-9]{1})");
	QProcess *route = new QProcess;
	
	route->start("route", QStringList() << "-n");
	route->waitForFinished();
	
	while (!route->atEnd())
	{
		QString line = route->readLine(150);
		line = line.trimmed();

		if (line.contains(addrRegExp))
		{
			QString dest = addrRegExp.cap(1);
			QString gate = addrRegExp.cap(2);
			QString genm = addrRegExp.cap(3);
			QString flag = addrRegExp.cap(4);
			QString metr = addrRegExp.cap(5);
			QString ref  = addrRegExp.cap(6);
			QString use  = addrRegExp.cap(7);
			QString ifac = addrRegExp.cap(8);

			logFile->write("  ");
			logFile->write(qPrintable(QString(dest).leftJustified(15)));
			logFile->write(qPrintable(QString(gate).leftJustified(15)));
			logFile->write(qPrintable(QString(genm).leftJustified(15)));
			logFile->write(qPrintable(QString(flag).leftJustified(10)));
			logFile->write(qPrintable(QString(metr).leftJustified(10)));
			logFile->write(qPrintable(QString(ref ).leftJustified(10)));
			logFile->write(qPrintable(QString(use ).leftJustified(10)));
			logFile->write(qPrintable(QString(ifac)));
			logFile->write("\n");
		}
	}
	
	logFile->write("\n\n");
	route = 0;
	delete route;
}

void GeneratorThread::getDNSServers()
{
	logFile->write("DNS servers:\n");
	
	QFile *resolvConf = new QFile("/etc/resolv.conf");
	
	if (resolvConf->open(QFile::ReadOnly))
	{
		int i = 1;
		QRegExp dnsRegExp("(nameserver\\s+)([0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3})\\s*$");
		
		while (!resolvConf->atEnd())
		{
			QString line = resolvConf->readLine(40);
			
			if (line.startsWith("nameserver") && line.contains(dnsRegExp))
			{
				QString dnsEntry = QString("  DNS server %1: ").arg(i);
				logFile->write(qPrintable(dnsEntry.leftJustified(30)));
				logFile->write(qPrintable(dnsRegExp.cap(2)));
				logFile->write("\n");
				i++;
			}
		}
		
		logFile->write("\n\n");
		
		resolvConf->close();
		resolvConf = 0;
		delete resolvConf;
	}
}

void GeneratorThread::getFirmwareInfo()
{
	QDir *dir = new QDir("/lib/firmware");
	QStringList content = dir->entryList(QStringList() << "*.bin*",QDir::Files|QDir::NoDotAndDotDot|QDir::NoSymLinks);
	QString entry;
	
	logFile->write("Firmware:\n");
	
	foreach (entry, content)
	{
		if (entry.startsWith("cxacru"))
		{
			logFile->write(qPrintable(QString("  Found cxacru firmware: ").leftJustified(30)));
			logFile->write(qPrintable(entry));
		}
		else if (entry.startsWith("speedtch"))
		{
			logFile->write(qPrintable(QString("  Found speedtch firmware: ").leftJustified(30)));
			logFile->write(qPrintable(entry));
		}
		
		logFile->write("\n");
	}
	
	dir->setPath("/lib/firmware/ueagle-atm");
	content = dir->entryList(QStringList() << "*.fw" << "*.bin*", QDir::Files|QDir::NoDotAndDotDot|QDir::NoSymLinks);
	
	foreach (entry, content)
	{
		logFile->write(qPrintable(QString("  Found ueagle-atm firmware: ").leftJustified(30)));
		logFile->write(qPrintable(entry));
		
		logFile->write("\n");
	}
	
	logFile->write("\n\n");
	
	dir = 0;
	delete dir;
}

void GeneratorThread::getPermissions()
{
	logFile->write("Permissions:\n");
	
	QStringList files;
	files << "/etc/ppp/pap-secrets" << "/etc/ppp/chap-secrets" << "/etc/ppp/options" << "/etc/ppp/peers/ubudsl"
	      << "/etc/ppp/peers/ubudsl_unicorn" << "/etc/init.d/ubudsld";
	
	QString file;
	foreach (file, files)
	{
		QFile f(file);
		QFileInfo fi(f);
		logFile->write("  ");
		logFile->write(qPrintable(f.fileName()));
		logFile->write(":\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ReadOwner: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ReadOwner) ? "Yes\n" : "No\n");

		logFile->write("    ");
		logFile->write(qPrintable(QString("WriteOwner: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::WriteOwner) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ExeOwner: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ExeOwner) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ReadUser: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ReadUser) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("WriteUser: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::WriteUser) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ExeUser: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ExeUser) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ReadGroup: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ReadGroup) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("WriteGroup: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::WriteGroup) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ExeGroup: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ExeGroup) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ReadOther: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ReadOther) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("WriteOther: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::WriteOther) ? "Yes\n" : "No\n");
		
		logFile->write("    ");
		logFile->write(qPrintable(QString("ExeOther: ").leftJustified(14)));
		logFile->write(fi.permission(QFile::ExeOther) ? "Yes\n" : "No\n");
		logFile->write("\n");
	}
	
	logFile->write("\n\n");
}

void GeneratorThread::setGatheredInfo(const QString &gatheredInfo)
{
	m_gatheredInfo = gatheredInfo;
}

void GeneratorThread::getKernelMessages()
{
	const int len = 20000;
	char *buf = new char[len];
	
	if ( klogctl(3, buf, len) <= 0 )
	{
		return;
	}
	
	QString log(buf);
	delete buf;
	
	if ( kernelMsgs->open(QFile::WriteOnly|QFile::Truncate) )
	{
		kernelMsgs->write( qPrintable(log.replace(QRegExp("\n<\\d>"), "\n")) );
		kernelMsgs->close();
	}
}
