#include "LogGenerator.h"


LogGenerator::LogGenerator(QWidget *parent)
    : QWizard(parent)
{
    q1 = tr("Your modem does <b>not</b> synchronize at all and you can not connect. True?");
    q2 = tr("Your system has been updated and the connection stopped working. True?");
    q3 = tr("You can not connect, but on another system on the same computer the connection works properly. True?");
    q4 = tr("You can connect and browse web pages but it (browsing) is very slow. True?");
    q5 = tr("You can connect but you can <b>not</b> browse web pages at all. True?");
    
    const int questions = 5;
    
    gThread = new GeneratorThread();
    
    pageQ1 = new QuestionYesNoPage(q1, 1, questions);
    pageQ2 = new QuestionYesNoPage(q2, 2, questions);
    pageQ3 = new QuestionYesNoPage(q3, 3, questions);
    pageQ4 = new QuestionYesNoPage(q4, 4, questions);
    pageQ5 = new QuestionYesNoPage(q5, 5, questions);
    
    summaryPage = new SummaryPage();
    reviewPage  = new ReviewPage();
    finishPage  = new FinishPage(button(QWizard::FinishButton), gThread);
    
    setWindowTitle(tr("UbuDSL Log Generator"));
    
	setPage(Intro_Page, new IntroPage());
	setPage(Q1_Page, pageQ1);
	setPage(Q2_Page, pageQ2);
	setPage(Q3_Page, pageQ3);
	setPage(Q4_Page, pageQ4);
	setPage(Q5_Page, pageQ5);
	setPage(Summary_Page, summaryPage);
	setPage(Review_Page, reviewPage);
	setPage(Finish_Page,  finishPage);
	
	setWizardStyle(QWizard::MacStyle);
    setPixmap(QWizard::BackgroundPixmap, QPixmap(":images/semitransparent_logo.png"));
    setOptions(QWizard::NoBackButtonOnStartPage|QWizard::CancelButtonOnLeft);
    
    setButtonText(QWizard::BackButton,   tr("&Back"));
    setButtonText(QWizard::NextButton,   tr("&Next"));
    setButtonText(QWizard::CancelButton, tr("&Cancel"));
    setButtonText(QWizard::FinishButton, tr("&Finish"));

    connect(this, SIGNAL(currentIdChanged(int)), this, SLOT(currentIdChanged(int)));
    connect(summaryPage, SIGNAL(textChanged()), this, SLOT(summaryChanged()));
}

QString LogGenerator::getGatheredInfo()
{
	QString gatheredInfo = "";
	
	gatheredInfo += "Answer 1: ";
	gatheredInfo += pageQ1->getYesOrNo() ? "Yes.\n" : "No.\n";
		
	gatheredInfo += "Answer 2: ";
	gatheredInfo += pageQ2->getYesOrNo() ? "Yes.\n" : "No.\n";

	gatheredInfo += "Answer 3: ";
	gatheredInfo += pageQ3->getYesOrNo() ? "Yes.\n" : "No.\n";
	
	gatheredInfo += "Answer 4: ";
	gatheredInfo += pageQ4->getYesOrNo() ? "Yes.\n" : "No.\n";
	
	gatheredInfo += "Answer 5: ";
	gatheredInfo += pageQ5->getYesOrNo() ? "Yes.\n" : "No.\n";
		
	gatheredInfo += "\n\n";
		
	gatheredInfo += "Summary:\n";
	
	
	QStringList summary = summaryPage->getSummary().split("\n");
	QString line;
	
	foreach (line, summary)
	{
		if (line.isEmpty())
			continue;
		
		while (line.count() > 50 && line.lastIndexOf(" ", 60) > -1)
		{
			QString tmp = line;
			tmp = tmp.remove(tmp.lastIndexOf(" ", 50), tmp.count());
			line = line.remove(0, line.lastIndexOf(" ", 50)+1);
			
			gatheredInfo += "  ";
			gatheredInfo += tmp;
			gatheredInfo += "\n";
		}
		
		gatheredInfo += "  ";
		gatheredInfo += line;
		gatheredInfo += "\n";
	}
	
	gatheredInfo += "\n\n";
	
	return gatheredInfo;
}

void LogGenerator::currentIdChanged(int id)
{
	if (id == Summary_Page)
	{
		button(QWizard::NextButton)->setDisabled(summaryPage->getSummary().isEmpty());
	}
	else if (id == Finish_Page)
	{
		setOptions(QWizard::NoCancelButton|QWizard::NoBackButtonOnLastPage);
		gThread->setGatheredInfo(getGatheredInfo());
		gThread->start();
	}
	else if (id == Review_Page)
	{
		QMap<QString, bool> map;
		map.insert(q1, pageQ1->getYesOrNo());
		map.insert(q2, pageQ2->getYesOrNo());
		map.insert(q3, pageQ3->getYesOrNo());
		map.insert(q4, pageQ4->getYesOrNo());
		map.insert(q5, pageQ5->getYesOrNo());
		
		reviewPage->updateData( map );
	}
}

LogGenerator::~LogGenerator()
{
	delete gThread;
	
	delete pageQ1;
	delete pageQ2;
	delete pageQ3;
	delete pageQ4;
	delete pageQ5;
	delete summaryPage;
	delete finishPage;
}

void LogGenerator::summaryChanged()
{
	if (currentId() == Summary_Page)
	{
		button(QWizard::NextButton)->setDisabled(summaryPage->getSummary().isEmpty());
	}
}

IntroPage::IntroPage(QWidget *parent)
    : QWizardPage(parent)
{
    setTitle(tr("Introduction"));

    labelTop = new QLabel(tr("This wizard will help you generate "
                             "a log. All you have to do is answer "
                             "to few questions."));
    labelTop->setWordWrap(true);

    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(labelTop);

    setLayout(layout);
}

ReviewPage::ReviewPage(QWidget *parent)
    : QWizardPage(parent)
{
    setTitle(tr("Review"));

    labelTop = new QLabel(tr("Below are placed your answers to "
    		                 "the questions. If you want to change "
    		                 "any answer, go back."));
    labelTop->setWordWrap(true);
    
    textBrowser = new QTextBrowser();

    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(labelTop);
    layout->addWidget(textBrowser);

    setLayout(layout);
}

void ReviewPage::updateData( QMap<QString, bool> &map )
{
	QMapIterator<QString, bool> it(map);
	textBrowser->clear();
	
	while ( it.hasNext() )
	{
		it.next();
		textBrowser->append(QString("<b>%1:</b> %2").arg(tr("Question"), it.key()));
		textBrowser->append(QString("<b>%1:</b> %2.").arg(tr("Answer"), (it.value() ? tr("Yes") : tr("No"))) );
		textBrowser->append("");
	}
}

SummaryPage::SummaryPage(QWidget *parent)
    : QWizardPage(parent)
{
    setTitle(tr("Summary"));

    labelTop = new QLabel(tr("Here you can describe your problem in details "
    		                 "or, if you want to, express an opinion about UbuDSL."));
    labelTop->setWordWrap(true);
    
    QString stringLng = tr("Please write in English or in Polish.");
    
    labelLng = new QLabel(QString("<i><b>%1</b></i>").arg(stringLng));
    labelLng->setTextFormat(Qt::RichText);
    labelLng->setWordWrap(true);

    textEdit = new QTextEdit();
    textEdit->setLineWrapMode(QTextEdit::WidgetWidth);
    connect(textEdit, SIGNAL(textChanged()), this, SIGNAL(textChanged()));
    
    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(labelTop);
    layout->addWidget(labelLng);
    layout->addWidget(textEdit);

    setLayout(layout);
}

QuestionYesNoPage::QuestionYesNoPage(const QString &question,
		                             const int &currQuestion,
		                             const int &questionsCount,
		                             QWidget *parent): QWizardPage(parent)
{
    setTitle(QString(tr("Question %1 of %2")).arg(currQuestion).arg(questionsCount));

    labelTop = new QLabel(question);
    labelTop->setWordWrap(true);
    
    radioYes = new QRadioButton(tr("Yes."));
    radioNo  = new QRadioButton(tr("No."));
    radioYes->setChecked(true);

    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(labelTop);
    layout->addWidget(radioYes);
    layout->addWidget(radioNo);

    setLayout(layout);
}

FinishPage::FinishPage(QAbstractButton *buttonFinish, GeneratorThread *gThread, QWidget *parent)
    : QWizardPage(parent)
{
    setTitle(tr("Finish"));
    
    this->buttonFinish = buttonFinish;
    this->gThread = gThread;
    buttonFinish->setDisabled(true);

    labelTop = new QLabel(tr("After log is already generated, a message will be shown above."));
    labelTop->setWordWrap(true);
    
    labelFinish = new QLabel();
    labelFinish->setWordWrap(true);
    
    labelFailed = new QLabel();
    labelFailed->setWordWrap(true);

    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(labelTop);
    layout->addWidget(labelFinish);
    layout->addWidget(labelFailed);

    setLayout(layout);

    connect(gThread, SIGNAL(finished()), this, SLOT(generatingFinished()));
    connect(gThread, SIGNAL(writingFailed()), this, SLOT(generatingFailed()));
}

void FinishPage::generatingFinished()
{
	QString msg = tr("Log has been successfully generated in <u>%2</u>.");
	
	labelFinish->setText(QString("<b><i>%1</b></i>").arg(msg).arg(gThread->getLogFilePath()));
	buttonFinish->setEnabled(true);
}

void FinishPage::generatingFailed()
{
	QString msg  = tr("Failed to generate log in <u>%2</u>.");
	QString msg2 = tr("Press <i>Finish</i> to quit.");
	
	labelFinish->hide();
	labelFailed->setText(QString("<b><i><font color=\"red\">%1</font>\n%3</b></i>").arg(msg).arg(gThread->getLogFilePath()).arg(msg2));
	buttonFinish->setEnabled(true);
}
