/*
 * File: Apt.cpp
 * Created: Fri 07 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "Apt.h"
#include <iostream>

extern bool  debug;
extern FILE *pFile;

Apt::Apt( QThread *parent )
: QObject(parent),
  m_cdProcess(new MyProcess(0)),
  m_installProcess(new MyProcess(0)),
  m_readListProcess(new QProcess(0)),
  m_aptCacheProcess(new QProcess(0)),
  m_unavailableRegExp( "W: Unable to locate package (.{1,100})" )
{
    m_cdProcess = new MyProcess(0);
    m_installProcess = new MyProcess(0);
    
    if ( parent != NULL )
    {
    	m_cdProcess->moveToThread(parent);
    	m_installProcess->moveToThread(parent);
    	m_readListProcess->moveToThread(parent);
    	m_aptCacheProcess->moveToThread(parent);
    }
    else
    {
    	if ( debug )
    		fprintf( pFile, "Apt::Apt(): parent is null!\n");
    }
    
    
    QStringList env = QProcess::systemEnvironment();
    env.append ( "LC_ALL=C" );

    m_cdProcess->setEnvironment ( env );
    m_installProcess->setEnvironment ( env );
    m_aptCacheProcess->setEnvironment ( env );

    connect ( m_cdProcess, SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugProcess ( QByteArray ) ) );
    connect ( m_installProcess,  SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugProcess ( QByteArray ) ) );
    connect ( m_aptCacheProcess, SIGNAL(lineAvailable(QByteArray)), this, SLOT(debugAptCache(QByteArray)));
}


Apt::~Apt()
{
	if ( debug )
		fprintf( pFile, "Apt::~Apt(): dtor called.\n");
	
	delete m_cdProcess;
	delete m_installProcess;
	delete m_readListProcess;
	delete m_aptCacheProcess;
}


void Apt::debugProcess ( QByteArray ba )
{
	if ( debug )
		fprintf( pFile, "Apt::debugProcess(): %s.\n", ba.constData()) ;
	
    dbg += ba;
}

int Apt::install ( const QStringList &pkgs, bool simulate )
{
    if ( pkgs.isEmpty() )
    {
    	if ( debug )
    		fprintf( pFile, "Apt::install(): packages list is empty, return -1!\n" );
    	
        return -1;
    }

	if ( debug )
		fprintf( pFile, "Apt::install(): packages list: %s.\n", qPrintable(pkgs.join(", ")) );
    
    m_installProcess->clearArguments();
    m_installProcess->addArgument ( "apt-get" );

    if ( simulate )
        m_installProcess->addArgument ( "-s" );

    m_installProcess->addArgument ( "install" );
    m_installProcess->addArgument ( "-y" );
    m_installProcess->addArgument ( "--force-yes" );

    foreach ( QString pkg, pkgs )
    	m_installProcess->addArgument ( pkg );

    dbg += "Apt::install(): begin.\n";
    
	if ( debug )
		fprintf( pFile, "Apt::install(): starting the install process.\n");
    
    m_installProcess->start();
    m_installProcess->waitForFinished ( 5 * 60 * 1000 );
    
    int exitCode = m_installProcess->exitCode();

    if ( exitCode != 0 )
    {
        dbg += "InstallProcess finished with code: ";
        dbg += exitCode;
        dbg += "\n";
    }

    dbg += "Apt::install(): end.\n";
    
	if ( debug )
		fprintf( pFile, "Apt::install(): exit code: %i!\n", exitCode );
	
    return exitCode;
}


int Apt::addCD ( const QString &mntPoint )
{
	if ( debug )
		fprintf( pFile, "Apt::addCD(): mount point: %s\n", qPrintable(mntPoint) );
	
    m_cdProcess->clearArguments();
    m_cdProcess->addArgument ( "apt-cdrom" );

    if ( !mntPoint.isEmpty() && QFile::exists ( mntPoint ) )
    {
        m_cdProcess->addArgument ( "-d" );
        m_cdProcess->addArgument ( mntPoint );
    }

    m_cdProcess->addArgument ( "-m" );
    m_cdProcess->addArgument ( "add" );
    
	if ( debug )
		fprintf( pFile, "Apt::addCD(): starting the process.\n");
    
    m_cdProcess->start();
    m_cdProcess->waitForFinished();
    
	if ( debug )
		fprintf( pFile, "Apt::addCD(): the process exited with code %i.\n", m_cdProcess->exitCode() );

    return m_cdProcess->exitCode();
}


QStringList Apt::getNotInstalled ( QStringList pkgs )
{
    if ( pkgs.isEmpty() )
    {
    	if ( debug )
    		fprintf( pFile, "Apt::getNotInstalled(): packages list is empty!\n" );
    	
        return QStringList();
    }
    
	if ( debug )
		fprintf( pFile, "Apt::getNotInstalled(): starting the process.\n");
	
    m_readListProcess->start ( "dpkg", QStringList() << "-l" );

    QByteArray readOutput;

    if ( m_readListProcess->waitForFinished() )
    {
    	if ( debug )
    		fprintf( pFile, "Apt::getNotInstalled(): waiting for process finish.\n");
    	
        readOutput = m_readListProcess->readAll();
    }

    QList<QByteArray> pkgList = readOutput.split ( '\n' );
    QListIterator<QByteArray> it ( pkgList );

    while ( it.hasNext() )
    {
        QByteArray ba = it.next();
        foreach ( QString pkg, pkgs )
        {
            if ( ba.startsWith ( qPrintable ( QString ( "ii  " + pkg ) ) ) )
            {
            	if ( debug )
            		fprintf( pFile, "Apt::getNotInstalled(): package %s is installed.\n", qPrintable(pkg) );
            	
                pkgs.removeAll ( pkg );
            }
        }
    }

	if ( debug )
		fprintf( pFile, "Apt::getNotInstalled(): not installed packages: %s.\n", qPrintable(pkgs.join(", ")) );
 
    return pkgs;
}

bool Apt::getArePackagesAvailable ( const QStringList &pkgs )
{
	if ( debug )
		fprintf( pFile, "Apt::getArePackagesAvailable(): packages count: %i.\n", pkgs.count() );
	
	m_available = true;
	qDebug() << 1 << m_available;
    m_aptCacheProcess->start ( "apt-cache", QStringList() << "showpkg" << pkgs );

    QByteArray readOutput;
    
    if ( m_aptCacheProcess->waitForFinished() )
    {
    	if ( debug )
    		fprintf( pFile, "Apt::getArePackagesAvailable(): waiting for process finish.\n");
    	
        readOutput = m_aptCacheProcess->readAllStandardError();
    }
    
	if ( QString(readOutput).contains(m_unavailableRegExp) )
	{
		m_available = false;
		
		if ( debug )
			fprintf( pFile, "Apt::debugAptCache: package %s is unavailable.\n", qPrintable(m_unavailableRegExp.cap(1)) );
	}
    
    if ( debug )
    	fprintf( pFile, "Apt::getArePackagesAvailable(): returning %s\n", m_available ? "true" : "false" );
    
    return m_available;
}
