/*
 * File: Compiler.h
 * Created: Tue 29 Jan 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "Compiler.h"

extern bool  debug;
extern FILE *pFile;

Compiler::Compiler ( const ModemDriver driver, QThread *parent )
: QObject(parent),
  m_compileDriverProcess(new MyProcess(0)),
  m_cleanDriverProcess(new MyProcess(0)),
  m_installDriverProcess(new MyProcess(0)),
  m_extractDriverProcess(new MyProcess(0)),
  m_patchProcess(new QProcess(0))
{
	qDebug() << "Compiler::Compiler: ctor invoked...";
	if ( parent != NULL )
	{
		m_compileDriverProcess->moveToThread(parent);
		m_cleanDriverProcess->moveToThread(parent);
		m_installDriverProcess->moveToThread(parent);
		m_extractDriverProcess->moveToThread(parent);
		m_patchProcess->moveToThread(parent);
	}
	else
	{
    	if ( debug )
    		fprintf( pFile, "Compiler::Compiler(): parent is null!\n" );
	}
	
    QStringList env = QProcess::systemEnvironment();
    env << "LC_ALL=C";
    m_extractDriverProcess->setEnvironment( env );
	
    extractedDriverPath     = driver.extracted_driver_path;
    driverName              = driver.name;
    driverVersion           = driver.version;
    compiledDriverName      = driver.compiled_driver_name;
    extractionPath          = driver.extraction_path;
    driverBundlePath        = driver.driver_bundle_path;

    compilationArguments    = driver.compilation_arguments.split ( ", " );
    installationArguments   = driver.installation_arguments.split ( ", " );
    environmentVariables    = driver.environment_variables.split ( ", " );

    kernel           = UbuDSL::SystemInfo::getKernelRelease();
    backupPath       = "/tmp/ubudsl_drv_backup/" + kernel;
    backupDir        = new QDir ( backupPath );
    backedUpModules  = QStringList();

    compilerDebug.clear();
    backupList.clear();


    if ( extractedDriverPath.isEmpty() || driverName.isEmpty() || driverVersion.isEmpty() ||
            compiledDriverName.isEmpty() || driverBundlePath.isEmpty() )
    {
    	if ( debug )
    		fprintf( pFile, "Compiler::Compiler(): needed variables are empty!\n" );
    }
}


Compiler::~Compiler()
{
	if ( debug )
		fprintf( pFile, "Compiler::~Compiler(): dtor called\n" );

    //Prepare directory for log
    QDir *tmpDir = new QDir ( "/opt/ubudsl/logs" );
    if ( !tmpDir->exists() )
    {
    	if ( debug )
    		fprintf( pFile, "Compiler::~Compiler(): logs' dir does not exist, creating a ne one.\n" );
    	
        tmpDir->mkpath ( tmpDir->absolutePath() );
    }

    delete tmpDir;

    //Prepare log file and write debug into it
    QFile *compilerLog = new QFile ( QString ( "/opt/ubudsl/logs/%1-%2_compiler.log" ).arg ( driverName ).arg ( driverVersion ) );
    if ( compilerLog->open ( QFile::WriteOnly | QFile::Truncate ) )
    {
        compilerLog->write ( compilerDebug );
        compilerLog->close();
    }

    delete compilerLog;
    delete m_compileDriverProcess;
    delete m_installDriverProcess;
    delete m_extractDriverProcess;
    delete m_patchProcess;
}


void Compiler::debugDriverCompileProcess ( QByteArray ba )
{
    compilerDebug += ba + "\n";
	if ( debug )
		fprintf( pFile, "Compiler::debugDriverCompileProcess(): %s\n", ba.constData() );
}


void Compiler::debugDriverRemoveProcess ( QByteArray ba )
{
    if ( ba.count ( ":" ) == 2 )
        backupList.append ( ba );

    compilerDebug += ba + "\n";
    
	if ( debug )
		fprintf( pFile, "Compiler::debugDriverRemoveProcess(): %s\n", ba.constData() );
}


void Compiler::debugDriverInstallProcess ( QByteArray ba )
{
    compilerDebug += ba + "\n";
	if ( debug )
		fprintf( pFile, "Compiler::debugDriverInstallProcess(): %s\n", ba.constData() );
}


void Compiler::backupDriver()
{
    if ( compiledDriverName.isEmpty() )
    {
        qWarning() << "Compiler::backupDriver(): \"compiledDriverName\" is empty or NULL, aborting!";
        return;
    }

    if ( !backupDir->exists() )
    {
        if ( !backupDir->mkpath ( backupPath ) )
        {
            qDebug() << QString ( "Compiler::backupDriver(): could not create backup path (%1), aborting!" )
            .arg ( backupPath );
            return;
        }
    }
    else
    {
        QStringList dirContent = backupDir->entryList ( QDir::Files | QDir::NoSymLinks );
        foreach ( QString file, dirContent )
        {
            QFile::remove ( backupPath + "/" + file );
        }
    }

    FileManager *fm = new FileManager;
    fm->indexFiles ( "/lib/modules/" + kernel );

    QStringList modules = fm->getIndex();
    modules = modules.filter ( "/" + compiledDriverName );

    foreach ( QString module, modules )
    {
        QFileInfo *fi = new QFileInfo ( module );
        QString moduleBUPath = fi->absolutePath();
        moduleBUPath = moduleBUPath.remove ( "/lib/modules/" + kernel );
        moduleBUPath.prepend ( backupPath );

        if ( !backupDir->exists ( moduleBUPath ) )
            backupDir->mkpath ( moduleBUPath );

        if ( backupDir->exists ( moduleBUPath ) &&
                QFile::copy ( fi->absoluteFilePath(), moduleBUPath + "/" + fi->fileName() ) )
        {
            backedUpModules.append ( moduleBUPath + "/" + fi->fileName() );
            QFile::remove ( fi->absoluteFilePath() );
        }

        delete fi;
    }
}


void Compiler::restoreDriver()
{
    foreach ( QString module, backedUpModules )
    {
        if ( QFile::exists ( module ) )
        {
            QString dest = module;
            dest = dest.remove ( 0, backupPath.count() );
            dest.prepend ( "/lib/modules/" + kernel + "/" );
            
            if ( QFile::copy ( module, dest ) )
                qDebug() << "Compiler::restoreDriver():" << module << "restored to" << dest;
            else
                qDebug() << "Compiler::restoreDriver():" << module << " could not be restored to" << dest;
        }
    }

    backedUpModules.clear();
}


int Compiler::cleanDriver()
{
    if ( extractedDriverPath.isEmpty() )
    {
        qWarning() << "Compiler::cleanDriver(): \"extractedDriverPath\" is empty or NULL, aborting!";
        return -1;
    }

    //Make sure that package was extracted into the correct dir
    QDir *tmpDir = new QDir ( extractedDriverPath );
    if ( !tmpDir->exists() )
    {
        qWarning() << "Compiler::cleanDriver(): extractedDriverPath: directory does not exist, aborting!";
        delete tmpDir;
        return -1;
    }

    delete tmpDir;

    m_cleanDriverProcess->clearArguments();
    m_cleanDriverProcess->setWorkingDirectory ( extractedDriverPath );

    QStringList env = QProcess::systemEnvironment();
    
    if ( !environmentVariables.isEmpty() )
        env += environmentVariables;
    
    m_cleanDriverProcess->setEnvironment ( env );

    m_cleanDriverProcess->addArgument ( "make" );
    m_cleanDriverProcess->addArgument ( "clean" );
    m_cleanDriverProcess->addArgument ( "-i" );
    

    connect ( m_cleanDriverProcess, SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugDriverCompileProcess ( QByteArray ) ) );

    compilerDebug += "# Begin Of Cleaning\n";

	if ( debug )
		fprintf( pFile, "Compiler::cleanDriver(): starting the process.\n");
	
	m_cleanDriverProcess->start();
	m_cleanDriverProcess->waitForFinished();

    int exitCode = m_cleanDriverProcess->exitCode();

	if ( debug )
		fprintf( pFile, "Compiler::cleanDriver(): the process exited with code %i.\n", exitCode );

    compilerDebug.append ( QString ( "\n# Process exited with status: %1.\n" ).arg ( exitCode ) );
    compilerDebug += "# End Of Cleaning\n\n\n\n";

    return exitCode;
}

int Compiler::compileDriver()
{
    if ( extractedDriverPath.isEmpty() )
    {
        qWarning() << "Compiler::compileDriver(): \"extractedDriverPath\" is empty or NULL, aborting!";
        return -1;
    }

    //Make sure that package was extracted into the correct dir
    QDir *tmpDir = new QDir ( extractedDriverPath );
    if ( !tmpDir->exists() )
    {
        qWarning() << "Compiler::compileDriver(): extractedDriverPath: directory does not exist, aborting!";
        delete tmpDir;
        return -1;
    }

    delete tmpDir;

    m_compileDriverProcess->clearArguments();
    m_compileDriverProcess->setWorkingDirectory ( extractedDriverPath );

    QStringList env = QProcess::systemEnvironment();
    
    if ( !environmentVariables.isEmpty() )
        env += environmentVariables;
    
    m_compileDriverProcess->setEnvironment ( env );

    m_compileDriverProcess->addArgument ( "make" );
    
    if ( !compilationArguments.isEmpty() )
    {
        foreach ( QString argument, compilationArguments )
        m_compileDriverProcess->addArgument ( argument );
    }
    else
    {
        qDebug() << "Compiler::compileDriver(): no arguments for \"make\"";
    }

    connect ( m_compileDriverProcess, SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugDriverCompileProcess ( QByteArray ) ) );

    compilerDebug += "# Begin Of Compilation\n";

	if ( debug )
		fprintf( pFile, "Compiler::compileDriver(): starting the process.\n");
	
    m_compileDriverProcess->start();
    m_compileDriverProcess->waitForFinished();

    int exitCode = m_compileDriverProcess->exitCode();

	if ( debug )
		fprintf( pFile, "Compiler::compileDriver(): the process exited with code %i.\n", exitCode );

    compilerDebug.append ( QString ( "\n# Process exited with status: %1.\n" ).arg ( exitCode ) );
    compilerDebug += "# End Of Compilation\n\n\n\n";

    return exitCode;
}


void Compiler::removeDriver()
{/*
 //Make sure the driver name is set, otherwise nothing to do
    if ( compiledDriverName.isEmpty() )
    {
        qWarning() << "Compiler::removeDriver(): \"compiledDriverName\" is empty or NULL, aborting!";
        return;
    }

    MyProcess *driverRemoveProcess = new MyProcess;
    driverRemoveProcess->clearArguments();
    driverRemoveProcess->addArgument( "/opt/ubudsl/bin/remove_driver" );
    driverRemoveProcess->addArgument( compiledDriverName );

    connect( driverRemoveProcess, SIGNAL(lineAvailable(QByteArray)),
             this,                SLOT  (debugDriverRemoveProcess(QByteArray)) );

    compilerDebug += "# Begin Of Removal\n";

    driverRemoveProcess->start();
    driverRemoveProcess->waitForFinished();

    compilerDebug.append( QString("\n# Process exited with status: %1.\n").arg(driverRemoveProcess->exitCode()) );
    compilerDebug += "# End Of Removal\n\n\n\n";

    delete driverRemoveProcess;*/
}


int Compiler::installDriver()
{
    if ( extractedDriverPath.isEmpty() )
    {
        qWarning() << "Compiler::installDriver(): \"extractedDriverPath\" is empty or NULL, aborting!";
        return -1;
    }

    //And again... Make sure the extracted driver is there, where should be
    QDir *tmpDir = new QDir ( extractedDriverPath );
    if ( !tmpDir->exists() )
    {
        qWarning() << "Compiler::installDriver(): extractedDriverPath: directory does not exist, aborting!";
        delete tmpDir;
        return -1;
    }

    delete tmpDir;

    m_installDriverProcess->clearArguments();
    m_installDriverProcess->setWorkingDirectory ( extractedDriverPath );

    QStringList env = QProcess::systemEnvironment();
    if ( !environmentVariables.isEmpty() )
        env += environmentVariables;
    m_installDriverProcess->setEnvironment ( env );

    m_installDriverProcess->addArgument ( "make" );

    if ( !installationArguments.isEmpty() )
    {
        foreach ( QString argument, installationArguments )
        m_installDriverProcess->addArgument ( argument );
    }
    else
    {
        qDebug() << "Compiler::installDriver(): no arguments for \"make\", using default.";
        m_installDriverProcess->addArgument ( "install" );
    }

    connect ( m_installDriverProcess, SIGNAL ( lineAvailable ( QByteArray ) ),
              this,                 SLOT ( debugDriverInstallProcess ( QByteArray ) ) );

    compilerDebug += "# Begin Of Installation\n";

	if ( debug )
		fprintf( pFile, "Compiler::installDriver(): starting the process.\n");
    
    m_installDriverProcess->start();
    m_installDriverProcess->waitForFinished();
    int exitCode = m_installDriverProcess->exitCode();

	if ( debug )
		fprintf( pFile, "Compiler::installDriver(): the process exited with code %i.\n", exitCode );
    
    compilerDebug.append ( QString ( "\n# Process exited with status: %1.\n" ).arg ( exitCode ) );

    if ( exitCode != 0 )
    {
        compilerDebug += "# RESTORING DRIVER\n";
        foreach ( QString data, backupList )
        {
            QString destPath = data;
            QString destName = data;
            QString srcPath  = data;
            destPath = destPath.remove ( destPath.indexOf ( ":" ), destPath.count() );
            destName = destName.remove ( 0, destName.indexOf ( ":" ) + 1 );
            destName = destName.remove ( destName.indexOf ( ":" ), destName.count() );
            srcPath  = srcPath .remove ( 0, srcPath.lastIndexOf ( ":" ) + 1 );

            qDebug() << destPath << destName << srcPath;
        }
        compilerDebug += "# END OF RESTORING DRIVER\n";
    }

    compilerDebug += "# End Of Installation\n\n\n\n";

    QProcess::execute ( "rm", QStringList() << "-rf" << extractedDriverPath );

    return exitCode;
}


void Compiler::extractDriver()
{
    if ( driverName.isEmpty() || driverVersion.isEmpty() || extractionPath.isEmpty() || driverBundlePath.isEmpty() )
    {
        qWarning() << "Compiler::extractDriver(): \"driverName\" and/or \"driverVersion\" and/or \"extractionPath\" and/or \"driverBundlePath\" is empty or NULL, aborting!";
        return;
    }

    if ( !QFile::exists ( driverBundlePath ) )
    {
        qWarning() << "Compiler::extractDriver(): driverBundlePath: file does not exist, aborting!";
        return;
    }

    //Remove old extracted directory if exists
    QProcess::execute ( "rm", QStringList() << "-rf" << extractedDriverPath );

    m_extractDriverProcess->clearArguments();
    m_extractDriverProcess->setWorkingDirectory ( extractedDriverPath );

    m_extractDriverProcess->addArgument ( "tar" );
    m_extractDriverProcess->addArgument ( "-v" );
    m_extractDriverProcess->addArgument ( "-C" );
    m_extractDriverProcess->addArgument ( extractionPath );
    m_extractDriverProcess->addArgument ( "-xf" );
    m_extractDriverProcess->addArgument ( driverBundlePath );

    connect ( m_extractDriverProcess, SIGNAL ( lineAvailable ( QByteArray ) ),
              this,                   SLOT ( debugDriverCompileProcess ( QByteArray ) ) );

    compilerDebug += "# Begin Of Extraction\n";

    m_extractDriverProcess->start();
    m_extractDriverProcess->waitForFinished();

    compilerDebug.append ( QString ( "\n# Process exited with status: %1.\n" ).arg ( m_extractDriverProcess->exitCode() ) );
    compilerDebug += "# End Of Extraction\n\n\n\n";
    
    cleanDriver();
}

void Compiler::patchDriver()
{
	if ( extractedDriverPath.isEmpty() )
	{
		qDebug() << "Compiler::patchDriver(): extractedDriverPath is empty, returning!";
		return;
	}
	
	QRegExp kernelRegExp ( "([0-9]{1})\\.([0-9]{1})\\.([0-9]{1,2})" );
    int kernelNum = 0;

    if ( kernel.contains ( kernelRegExp ) )
    {
        QString str = kernelRegExp.cap ( 1 );
        str += kernelRegExp.cap ( 2 );
        str += kernelRegExp.cap ( 3 );

        kernelNum = str.toInt();
    }
    
    QFileInfo patchFile(driverBundlePath);
    
    patchFile.setFile( QString( "%1/%2-%3_%4.patch" ).arg(patchFile.absolutePath())
    												 .arg(driverName)
    												 .arg(driverVersion)
    												 .arg(kernel));
    
    if ( patchFile.exists() && patchFile.isFile() )
    {
    	qDebug() << "Compiler::patchDriver(): Patching" << driverName << "by" << patchFile.absoluteFilePath();
    	
    	m_patchProcess->setWorkingDirectory(extractedDriverPath);
    	m_patchProcess->start( "patch", QStringList() << "-p0" << patchFile.absoluteFilePath() );
    	m_patchProcess->waitForFinished(5000);
    	
    	qDebug() << "Compiler::patchDriver(): patch process exited with code:" << m_patchProcess->exitCode();
    }
}
