/*
 * File: DNS_Manager.cpp
 * Created: Fri 22 Feb 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "DNS_Manager.h"

extern bool  debug;
extern FILE *pFile;


DNS_Manager::DNS_Manager()
        : m_resolvConf ( new QFile ( "/etc/resolv.conf" ) ),
        m_oldResolvConf ( new QFile ( "/etc/resolv.conf_ubudsl" ) )
{
	if ( debug )
		fprintf( pFile, "DNS_Manager::DNS_Manager(): ctor called.\n" );
	
    QString octet ( "(([0-9]{1,2})|(1{1}[0-9]{2})|(2{1}[5]{1}[0-5]{1})|(2{1}[0-4]{1}[0-9]{1}))" );
    m_ipRegExp.setPattern ( "^" + octet + "\\." +  octet + "\\." +  octet + "\\." +  octet + "$" );
    m_resolvConfEntry.setPattern ( "^nameserver " + octet + "\\." +  octet + "\\." +  octet + "\\." +  octet + "\\s*$" );
}

bool DNS_Manager::setServers ( QStringList &addresses )
{
	bool ret = false;

	if ( debug )
		fprintf( pFile, "DNS_Manager::setServers(): %s\n", qPrintable(addresses.join(", ")) );

    foreach ( QString address, addresses )
    {
        if ( !m_ipRegExp.exactMatch ( address ) )
            addresses.removeAll ( address );
    }

    if ( addresses.isEmpty() )
    {
    	if ( debug )
    		fprintf( pFile, "DNS_Manager::setServers(): No (valid) DNS server's addresses were given, aborting!\n" );

        return false;
    }

    //back up resolv.conf if UbuDSL didn't do this before
    if ( !m_oldResolvConf->exists() )
    {
        if ( m_resolvConf->copy ( m_oldResolvConf->fileName() ) )
            qDebug() << "UbuDSL::setDNS(): resolv.conf backed up to" << m_oldResolvConf->fileName();
        else
            qDebug() << "UbuDSL::setDNS(): could NOT back up resolv.conf to" << m_oldResolvConf->fileName();
    }

    m_resolvConf->close(); //just to be sure

    if ( m_resolvConf->open ( QFile::WriteOnly | QFile::Truncate ) )
    {
        qDebug() << "valid addresses:" << addresses;
        ret = true;

        foreach ( QString address, addresses )
        {
            m_resolvConf->write ( "nameserver " );
            m_resolvConf->write ( qPrintable ( address ) );
            m_resolvConf->write ( "\n" );
        }

        m_resolvConf->close();
        qDebug() << "UbuDSL::setDNS(): The DNS servers" << addresses << "have been written.";
    }
    else
    {
        qDebug() << "UbuDSL::setDNS(): could not open" << m_resolvConf->fileName() << "in WriteOnly mode!";
    }
    
    return ret;
}

bool DNS_Manager::restore()
{
    if ( !m_oldResolvConf->exists() )
        return false;

    m_resolvConf->remove();

    if ( m_resolvConf->exists() && debug )
        fprintf( pFile, "UbuDSL::restoreOldDNS(): %s was not removed, cannot restore the old one!", qPrintable(m_resolvConf->fileName()) );

    if ( m_oldResolvConf->copy ( m_resolvConf->fileName() ) )
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL::restoreOldDNS(): %s restored successfully.", qPrintable(m_resolvConf->fileName()) );

        return true;
    }
    else
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL::restoreOldDNS(): %s could not be restored!", qPrintable(m_resolvConf->fileName()) );

        return false;
    }
}

bool DNS_Manager::isFileCorrect()
{
    if ( m_resolvConf->open ( QFile::ReadOnly ) )
    {
        while ( !m_resolvConf->atEnd() )
        {
            QString line = m_resolvConf->readLine ( 100 );

            if ( (!line.startsWith ( "#" ) && !m_resolvConfEntry.exactMatch ( line ) && !line.trimmed().isEmpty()) || line.startsWith("# Modified_by:  NetworkManager") )
                return false;
        }

        m_resolvConf->close();
    }
    else
    {
        return false;
    }

    return true;
}
