/*
 * File: FileManager.h
 * Created: Thu 31 Jan 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v. 3 or newer
 */

#include "FileManager.h"

FileManager::FileManager()
{
    mAllFiles = QStringList();
    depth = 0;
}


FileManager::~FileManager()
{
    qDebug() << "~FileManager()";
}


void FileManager::copyFiles ( const QStringList &filesToCopy, const QString &destPath )
{
    //Make sure the destination path exists and create otherwise
    QDir *destDir = new QDir ( destPath );
    if ( !destDir->exists() && !destDir->mkpath ( destDir->absolutePath() ) )
    {
        qWarning() << "FileManager::copyFiles(): Destination directory does not exist and can not be created, aborting!";
        return;
    }

    //Copy all the files, one by one
    foreach ( const QString fileName, filesToCopy )
    {
        QFileInfo *fileInfo = new QFileInfo ( fileName );
        if ( QFile::copy ( fileName, destPath + "/" + fileInfo->fileName() ) )
            qDebug() << fileName << "copied.";
        else
            qDebug() << "could not copy" << fileName << "to" << destPath + "/" + fileInfo->fileName();

        delete fileInfo;
    }
}


void FileManager::copyFile ( const QString &fileToCopy, const QString &destPath )
{
    //Make sure the destination path exists and create otherwise
    QDir *destDir = new QDir ( destPath );
    if ( !destDir->exists() && !destDir->mkpath ( destDir->absolutePath() ) )
    {
        qWarning() << "FileManager::copyFiles(): Destination directory does not exist and can not be created, aborting!";
        return;
    }

    QFileInfo *fileInfo = new QFileInfo ( fileToCopy );
    if ( QFile::copy ( fileToCopy, destPath + "/" + fileInfo->fileName() ) )
        qDebug() << fileToCopy << "copied.";
    else
        qDebug() << "could not copy" << fileToCopy << "to" << destPath + "/" + fileInfo->fileName();

    delete fileInfo;
}


void FileManager::findFile ( const QString rootDir, const QString &fileName, QString &foundFile )
{
    //Make sure the dir in which we will be searching exists
    QDir *dir = new QDir ( rootDir );
    if ( !dir->exists() )
        return;

    //qDebug() << "Searching in directory:" << rootDir;

    //List only files in current dir. No symbolic links.
    QStringList filesList ( dir->entryList ( QDir::Files | QDir::NoSymLinks ) );
    foreach ( QString file, filesList )
    {
        //If 'fileName' is the same as the name of the
        //current file, we add the file with whole path
        //to the 'foundFile' string and return from this
        //function.
        if ( file == fileName )
        {
            qDebug() << "Found matching file:" << file;
            foundFile = dir->absolutePath() + "/" + file;
            return;
        }
    }

    //List all the subdirectories in the current dir and run this function
    //for each subfolder recursively
    QStringList dirsList ( dir->entryList ( QDir::Dirs | QDir::NoDotAndDotDot ) );
    foreach ( QString directory, dirsList )
    {
        findFile ( dir->absolutePath() + "/" + directory, fileName, foundFile );
    }
}


void FileManager::findFiles ( const QString rootDir, const QRegExp &fileNamePattern, QStringList &foundFiles )
{
    //Make sure the dir in which we will be searching exists
    QDir *dir = new QDir ( rootDir );
    if ( !dir->exists() )
        return;

    qDebug() << "Searching in directory:" << rootDir;

    //List all the files in current dir (no symlinks)
    QStringList tmpAllFiles = ( dir->entryList ( QDir::Files | QDir::NoSymLinks ) );
    foreach ( QString entry, tmpAllFiles )
    {
        if ( !mAllFiles.contains ( rootDir + "/" + entry ) )
            mAllFiles.append ( rootDir + "/" + entry );
    }

    //Run this function recursively for all the subdirs
    QStringList dirsList ( dir->entryList ( QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks ) );
    foreach ( QString directory, dirsList )
    {
        findFiles ( dir->absolutePath() + "/" + directory, fileNamePattern, foundFiles );
    }

    foreach ( QString entry, mAllFiles )
    {
        //Add found files (avoiding duplicating) to the 'foundFiles' list
        if ( !foundFiles.contains ( entry ) && QFileInfo::QFileInfo ( entry ).fileName().contains ( fileNamePattern ) )
            foundFiles.append ( entry );
    }
}


bool FileManager::indexFiles ( const QString &rootDir )
{
    QString myPath = QFileInfo ( rootDir ).absoluteFilePath();
    if ( !QDir().exists ( myPath ) )
        return false;

    QFileInfoList list = QDir ( myPath ).entryInfoList ( QDir::AllDirs | QDir::Files | QDir::NoSymLinks
                         | QDir::NoDotAndDotDot | QDir::Readable | QDir::Hidden );

    for ( QFileInfoList::iterator p = list.begin(); p != list.end(); p++ )
    {
        if ( ( *p ).isDir() && ( ( *p ).absoluteFilePath() != myPath ) )
        {
            if ( !indexFiles ( ( *p ).absoluteFilePath() ) )
                return false;
        }
        else
        {
            if ( !QFile::exists ( ( *p ).absoluteFilePath() ) ) continue;

            mIndexedFiles << ( *p ).absoluteFilePath();
        }
    }

    return true;
}


void FileManager::removeFile ( const QString &fileName )
{
    QFile *tmpFile = new QFile ( fileName );
    if ( tmpFile->exists() )
    {
        if ( tmpFile->remove() )
            qDebug() << fileName << "removed.";
        else
            qDebug() << fileName << "could not be removed!";
    }
    else
        qDebug() << fileName << "does not exist, skipping.";

    delete tmpFile;
}


void FileManager::clearIndex()
{
    mIndexedFiles.clear();
}


void FileManager::clearFoundFilesList()
{
    mAllFiles.clear();
}


QStringList FileManager::getIndex()
{
    return mIndexedFiles;
}
