/*
 * File: MediaManager.h
 * Created: Wed 30 Jan 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "MediaManager.h"

//Connect to the DBus HAL interface
QDBusInterface hal_iface ( "org.freedesktop.Hal"        , "/org/freedesktop/Hal/Manager",
                           "org.freedesktop.Hal.Manager", QDBusConnection::systemBus() );

MediaManager::MediaManager( QThread *parent )
:QObject(parent),
 m_mountProcess(new QProcess(0)),
 m_unmountProcess(new QProcess(0)),
 m_ejectProcess(new QProcess(0))
{
	if ( parent != NULL )
	{
		m_mountProcess->moveToThread(parent);
		m_unmountProcess->moveToThread(parent);
		m_ejectProcess->moveToThread(parent);
	}
	
    QDBusConnection::systemBus().connect ( QString(), QString(),
                                           "org.freedesktop.Hal.Manager",
                                           "DeviceAdded", this,
                                           SLOT ( deviceAdded ( QString ) ) );
    QDBusConnection::systemBus().connect ( QString(), QString(),
                                           "org.freedesktop.Hal.Manager",
                                           "DeviceRemoved", this,
                                           SLOT ( deviceRemoved ( QString ) ) );
}


MediaManager::~MediaManager()
{
    qDebug() << "~MediaManager()";
    delete m_mountProcess;
    delete m_unmountProcess;
    delete m_ejectProcess;
}


QList<Volume> MediaManager::getVolumesList()
{
    m_volumesList.clear();
    
    QDBusReply<QStringList> reply = hal_iface.call ( "FindDeviceByCapability", "volume.disc" );
    if ( reply.isValid() )
    {
        foreach ( QString udi, reply.value() )
        {
            QDBusInterface iface ( "org.freedesktop.Hal", udi,
                                   "org.freedesktop.Hal.Device",
                                   QDBusConnection::systemBus() );
            QDBusReply<QString> device = iface.call ( "GetProperty", "block.device" );
            QDBusReply<QString> label  = iface.call ( "GetProperty", "volume.label" );

            if ( device.isValid() && label.isValid() )
            	m_volumesList.append( Volume(device.value(), label.value(), udi ) );
        }
    }

    qDebug() << "Volumes list count:" << m_volumesList.count();
    return m_volumesList;
}


int MediaManager::getVolumesCount()
{
    int volumesCount;

    QDBusReply<QStringList> reply = hal_iface.call ( "FindDeviceByCapability", "volume.disc" );
    if ( reply.isValid() )
    {
        volumesCount = reply.value().count();
    }
    else
    {
        volumesCount = 0;
    }

    return volumesCount;
}

QString MediaManager::getVolumeMountPoint ( const QString &udi )
{
    //Create an interface for current device (udi), check if it
    //is mounted, and if so, return the mount point
    QDBusInterface iface ( "org.freedesktop.Hal", udi,
                           "org.freedesktop.Hal.Device", QDBusConnection::systemBus() );
    QDBusReply<QString> mountPoint = iface.call ( "GetProperty", "volume.mount_point" );

    if ( mountPoint.isValid() )
    {
        qDebug() << "MediaManager::getVolumeMountPoint(): Returning mount point:" << mountPoint.value();
        return mountPoint.value();
    }
    else
    {
        qDebug() << "MediaManager::getVolumeMountPoint(): dbus reply is invalid. "
        "Returning empty string!";
        qDebug() << "MediaManager::getVolumeMountPoint(): dbus reply error:" << mountPoint.error();
        return QString();
    }
}

int MediaManager::mountVolume ( const QString &udi, const QString &mountPoint )
{
    if ( mountPoint.isEmpty() )
    {
        qDebug() << "MediaManager::mountVolume(): empty mount point, aborting (returning -1)!";
        return -1;
    }

    QDBusInterface iface ( "org.freedesktop.Hal", udi,
                           "org.freedesktop.Hal.Device", QDBusConnection::systemBus() );
    QDBusReply<QString> deviceFile = iface.call ( "GetProperty", "block.device" );

    if ( !deviceFile.isValid() )
    {
        qDebug() << "MediaManager::mountVolume(): reply is invalid, aborting (returning -2)";
        return -2;
    }

    QDir *mountDir = new QDir ( mountPoint );
    if ( !mountDir->exists() )
    {
        if ( !mountDir->mkpath ( mountPoint ) )
        {
            qDebug() << QString ( "MediaManager::mountVolume(): could not create "
                                  "mount path (%1), aborting (returning -3)" ).arg ( mountPoint );
            delete mountDir;
            return -3;
        }

        delete mountDir;
        qDebug() << QString ( "MediaManager::mountVolume(): mount path "
                              "\"%1\" successfully created." ).arg ( mountPoint );
    }

    m_mountProcess->start ( "mount", QStringList() << deviceFile.value() << mountPoint );

    if ( m_mountProcess->waitForFinished() )
    {
        int exitCode = m_mountProcess->exitCode();
        qDebug() << "MediaManager::mountVolume(): exit code:" << exitCode;

        return exitCode;
    }
    else
    {
        qDebug() << "MediaManager::mountVolume(): couldn't mount volume, returning -4!";
        return -4;
    }
}

int MediaManager::unmountVolume ( const QString &mountPoint, const bool forced = false )
{
    if ( mountPoint.isEmpty() )
    {
        qDebug() << "MediaManager::unmountVolume(): empty mount point, aborting (returning -1)!";
        return -1;
    }

    QStringList arguments;

    if ( forced ) arguments << "-l";

    m_unmountProcess->start ( "umount", arguments << mountPoint );

    if ( m_unmountProcess->waitForFinished ( 10000 ) )
    {
        int exitCode = m_unmountProcess->exitCode();
        qDebug() << "MediaManager::unmountVolume(): exit code:" << exitCode;

        return exitCode;
    }
    else
    {
        qDebug() << "MediaManager::unmountVolume(): couldn't umount volume, returning -2!";
        return -2;
    }
}

bool MediaManager::ejectVolume ( const QString &udi )
{
    unmountVolume ( getVolumeMountPoint ( udi ) );
    QDBusInterface iface ( "org.freedesktop.Hal", udi,
                           "org.freedesktop.Hal.Device", QDBusConnection::systemBus() );
    QDBusReply<QString> bDevice = iface.call ( "GetProperty", "block.device" );
    QStringList arguments;
    if ( bDevice.isValid() )
        arguments << bDevice.value();

    m_ejectProcess->start ( "eject", arguments );

    if ( m_ejectProcess->waitForFinished ( 10000 ) )
    {
        int exitCode = m_ejectProcess->exitCode();
        qDebug() << "MediaManager::unmountVolume(): exit code:" << exitCode;

        return ( exitCode == 0 ? true : false );
    }
    else
    {
        qDebug() << "MediaManager::unmountVolume(): couldn't eject volume, returning false!";

        return false;
    }
}

void MediaManager::deviceAdded ( const QString &device )
{
    QDBusInterface iface ( "org.freedesktop.Hal", device,
                           "org.freedesktop.Hal.Device", QDBusConnection::systemBus() );
    QDBusReply<bool> isVolume = iface.call ( "GetProperty", "volume.is_disc" );

    if ( isVolume.isValid() && isVolume.value() )
    {
        qDebug() << "Volume added...";
        QDBusReply<QString> bDevice = iface.call ( "GetProperty", "block.device" );
        QDBusReply<QString> label   = iface.call ( "GetProperty", "volume.label" );

        if ( bDevice.isValid() && label.isValid() )
        {
            qDebug() << "Volume ok:" << bDevice.value() << label.value();
            emit volumeAdded ( bDevice.value(), label.value(), device );
            emit volumeAdded();
        }
    }
}

void MediaManager::deviceRemoved ( const QString &device )
{
    qDebug() << "Volume removed:" << device;
    emit volumeRemoved ( device );
    emit volumeRemoved();
}

bool MediaManager::getVolumeIsMounted ( const QString &udi )
{
    //Create an interface for current device (udi), check if it
    //is mounted.
    QDBusInterface iface ( "org.freedesktop.Hal", udi,
                           "org.freedesktop.Hal.Device", QDBusConnection::systemBus() );
    QDBusReply<bool> isMounted = iface.call ( "GetProperty", "volume.is_mounted" );

    if ( isMounted.isValid() )
    {
        qDebug() << "MediaManager::getVolumeIsMounted(): Returning mount point:" << isMounted.value();
        return isMounted.value();
    }
    else
    {
        qDebug() << "MediaManager::getVolumeIsMounted(): dbus reply is invalid. "
        "Returning false!";
        qDebug() << "MediaManager::getVolumeIsMounted(): dbus reply error:" << isMounted.error();
        return false;
    }
}
