/*
 * File: SystemInfo.cpp
 * Created: Thu 31 Jan 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v. 3 or newer
 */

#include "SystemInfo.h"

namespace UbuDSL
{
namespace SystemInfo
{
QRegExp fedoraRegExp    ( "(^Fedora)\\s+[release]*\\s+([0-9]{1,2})\\s+" );
QRegExp pclinuxosRegExp ( "(^PCLinuxOS)\\s+release\\s+([0-9]{4})\\s+\\(PCLinuxOS\\)\\s+for" );
QRegExp debianRegExp    ( "(^[0-9]{1}\\.[0-9]\\s*$)" );

QString getKernelRelease()
{
    struct utsname buffer;

    if ( uname ( &buffer ) == 0 )
    {
        return buffer.release;
    }
    else
    {
        qDebug() << "getKernelRelease(): Could not get kernel release, returning empty string!";
        return QString();
    }
}

QString getArchitecture()
{
    struct utsname buffer;

    if ( uname ( &buffer ) == 0 )
    {
        QString arch = buffer.machine;
        if ( arch == "i386" ||
                arch == "i486" ||
                arch == "i586" ||
                arch == "i686"
           )
            arch = "i386";
        else
            arch = "amd64";
        return arch;
    }
    else
    {
        qDebug() << "getArchitecture(): Could not get architecture, returning empty string!";
        return QString();
    }
}

QString getProcessorArchitecture()
{
    struct utsname buffer;

    if ( uname ( &buffer ) == 0 )
    {
        QString arch = buffer.machine;
        return arch;
    }
    else
    {
        qDebug() << "getProcessorArchitecture(): Could not get architecture, returning empty string!";
        return QString();
    }
}

QString getDistributionName()
{
    QFile releaseFile;

    releaseFile.setFileName ( "/etc/SuSE-release" );
    if ( releaseFile.exists() )
    {
        return QString ( "SuSE" );
    }

    releaseFile.setFileName ( "/etc/redhat-release" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString redhatRelease = releaseFile.readLine ( 50 );
        releaseFile.close();

        if ( redhatRelease.contains ( fedoraRegExp ) )
            return fedoraRegExp.cap ( 1 );
        else if ( redhatRelease.contains ( pclinuxosRegExp ) )
        	return pclinuxosRegExp.cap ( 1 );
    }

    releaseFile.setFileName ( "/etc/lsb-release" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString content = releaseFile.readAll().data();
        releaseFile.close();
        foreach ( QString entry, content.split ( "\n" ) )
        {
            if ( entry.startsWith ( "DISTRIB_ID=Ubuntu" ) )
            {
                return QString ( "Ubuntu" );
            }
        }
    }

    releaseFile.setFileName ( "/etc/debian_version" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString debianRelease = releaseFile.readLine ( 50 );
        releaseFile.close();

        if ( debianRelease.startsWith( "lenny/sid" ) || debianRelease.contains ( debianRegExp ) )
            return "Debian";
    }

    qDebug() << QString ( "Could not open %1 or %1 does not contain proper data, returning empty string!" ).arg ( releaseFile.fileName() );
    return QString();
}

QString getDistributionRelease()
{
    QFile releaseFile;

    releaseFile.setFileName ( "/etc/SuSE-release" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString content = releaseFile.readAll().data();
        releaseFile.close();
        foreach ( QString entry, content.split ( "\n" ) )
        {
            if ( entry.startsWith ( "VERSION = " ) )
            {
                return entry.remove ( "VERSION = " );
            }
        }
    }

    releaseFile.setFileName ( "/etc/redhat-release" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString redhatRelease = releaseFile.readLine ( 50 );
        releaseFile.close();

        if ( redhatRelease.contains ( fedoraRegExp ) )
            return fedoraRegExp.cap ( 2 );
        else if ( redhatRelease.contains ( pclinuxosRegExp ) )
        	return pclinuxosRegExp.cap ( 2 );
    }

    releaseFile.setFileName ( "/etc/lsb-release" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString content = releaseFile.readAll().data();
        releaseFile.close();
        foreach ( QString entry, content.split ( "\n" ) )
        {
            if ( entry.startsWith ( "DISTRIB_RELEASE=" ) )
            {
                return entry.remove ( "DISTRIB_RELEASE=" );
            }
        }
    }

    releaseFile.setFileName ( "/etc/debian_version" );
    if ( releaseFile.exists() && releaseFile.open ( QFile::ReadOnly ) )
    {
        QString debianRelease = releaseFile.readLine ( 50 );
        releaseFile.close();

        if ( debianRelease.startsWith( "lenny/sid" ) )
            return "4.0";
        if ( debianRelease.contains ( debianRegExp ) )
            return debianRegExp.cap ( 1 );
    }

    qDebug() << QString ( "Could not open %1 or %1 does not contain proper data, returning empty string!" ).arg ( releaseFile.fileName() );
    return QString();
}
}//namespace SystemInfo
}//namespace UbuDSL
