/*
 * File: Zypper.cpp
 * Created: Apr 2008
 * Last modified: Wed 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#include "Zypper.h"

Zypper::Zypper( QThread *parent )
:Rpm(parent),
 m_installProcess(new MyProcess(0)),
 m_zypperProcess(new MyProcess(0)),
 m_addRepoProcess(new QProcess(0)),
 m_modifyRepoProcess(new QProcess(0)),
 m_getAvailableProcess(new QProcess(0))
{
    dbg = QByteArray();

    if ( parent != NULL )
    {
    	m_installProcess->moveToThread(parent);
    	m_zypperProcess->moveToThread(parent);
    	m_addRepoProcess->moveToThread(parent);
    	m_modifyRepoProcess->moveToThread(parent);
    	m_getAvailableProcess->moveToThread(parent);
    }
    
    QStringList env = QProcess::systemEnvironment();
    env.append ( "LC_ALL=C" );

    m_installProcess->setEnvironment( env );
    m_zypperProcess->setEnvironment( env );
    m_addRepoProcess->setEnvironment( env );
    m_modifyRepoProcess->setEnvironment( env );
    m_getAvailableProcess->setEnvironment( env );

    repoRegExp.setPattern ( "(\\b[0-9]{1,2}\\b)\\s*\\|\\s*(\\bYes|No\\b)\\s*\\|\\s*(\\bYes|No\\b)\\s*\\|\\s*(\\b.{1,10}\\b)\\s*\\|\\s*(\\b.{1,40}\\b)\\s*\\|\\s*(\\b.{1,40}\\b)" );
    
    QString readingCache( "(^. Reading repository .{1,100} cache)" );
    QString readingInstalled( "(^. Reading installed packages)" );
    QString installingPkg( "(^\\* Installing: )" );
    
    processing.setPattern( readingCache + "|" + readingInstalled + "|" + installingPkg );
    
    connect ( m_installProcess, SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugProcess ( QByteArray ) ) );
}


Zypper::~Zypper()
{
    delete m_installProcess;
    delete m_zypperProcess;
    delete m_addRepoProcess;
    delete m_modifyRepoProcess;
    delete m_getAvailableProcess;
}


void Zypper::debugProcess ( QByteArray ba )
{
	if ( QString( ba ).contains( processing ) )
		return;
	
    qDebug() << "Zypper::debugProcess():" << ba;
    dbg += ba;
}


void Zypper::debugZyppRepos ( QByteArray ba )
{
    QString line = ba;

    if ( line.contains ( repoRegExp ) )
    {
        ZypperRepo repo;
        repo.number  = repoRegExp.cap ( 1 ).toInt();
        repo.enabled = ( repoRegExp.cap ( 2 ) == "Yes" ? true : false );
        repo.refresh = ( repoRegExp.cap ( 3 ) == "Yes" ? true : false );
        repo.type    = repoRegExp.cap ( 4 );
        repo.alias   = repoRegExp.cap ( 5 );
        repo.name    = repoRegExp.cap ( 6 );

        repos.append ( repo );
    }
}


int Zypper::install ( const QStringList &pkgs )
{
    if ( pkgs.isEmpty() )
        return 0;

    m_installProcess->clearArguments();

    m_installProcess->addArgument ( "zypper" );
    m_installProcess->addArgument ( "--non-interactive" );
    m_installProcess->addArgument ( "--no-gpg-checks" );
// m_installProcess->addArgument( "--auto-agree-with-licenses" );
    m_installProcess->addArgument ( "install" );

    foreach ( QString pkg, pkgs )
    {
        m_installProcess->addArgument ( pkg );
    }

    dbg += "Zypper::install(): begin.";
    m_installProcess->start();
    m_installProcess->waitForFinished ( 5 * 60 * 1000 );
    dbg += "Zypper::install(): end.";

    int exitCode = m_installProcess->exitCode();
    qDebug() << "Zypper::install(): m_installProcess exited with code:" << exitCode;

    return exitCode;
}

int Zypper::addRepo ( const QString &path, const QString &alias )
{
    m_addRepoProcess->start ( "zypper", QStringList() << "ar" << path << alias );
    m_addRepoProcess->waitForFinished();

    return m_addRepoProcess->exitCode();
}

bool Zypper::enableRepo ( QString alias, bool enable )
{
    QString success ( "Repository " + alias + " has been sucessfully modified." );

    m_modifyRepoProcess->start ( "zypper", QStringList() << "mr" << ( enable ? "-e" : "-d" ) << alias );
    m_modifyRepoProcess->waitForFinished();

    if ( m_modifyRepoProcess->readAll().contains ( qPrintable ( success ) ) )
        return true;
    else
        return false;
}

bool Zypper::getArePackagesAvailable ( const QStringList &pkgs )
{
	m_getAvailableProcess->start ( "apt-cache", QStringList() << "show" << pkgs );
	m_getAvailableProcess->waitForFinished();

    QRegExp notAvailable ( "package (.{1,100}) not found" );

    while ( !m_getAvailableProcess->atEnd() )
    {
        QString line = m_getAvailableProcess->readLine ( 200 );
        if ( line.contains ( notAvailable ) )
            return false;
    }

    return true;
}

QList<ZypperRepo> Zypper::getRepos()
{
	m_zypperProcess->addArgument ( "zypper" );
	m_zypperProcess->addArgument ( "repos" );

    connect ( m_zypperProcess, SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugZyppRepos ( QByteArray ) ) );

    m_zypperProcess->start();
    m_zypperProcess->waitForFinished();

    disconnect ( m_zypperProcess, SIGNAL ( lineAvailable ( QByteArray ) ), this, SLOT ( debugZyppRepos ( QByteArray ) ) );

    return repos;
}
