/*
 * File: UbuDSL_Thread.cpp
 * Created: Thu 07 Feb 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#include "UbuDSL_Thread.h"

extern bool  debug;
extern FILE *pFile;

UbuDSL_Thread::UbuDSL_Thread ( const QString &login, const QString &password, const int &protocol,
                               const int &VPI, const int &VCI, const QString &service,
                               const QStringList &dnsServers,const bool &connectAtBoot,
                               const bool &connectAtModemPlug, const int lockedFD, EndState *endState
                             )
: fileManager ( new FileManager() ),
  scriptManager ( new ScriptManager() ),
  modemManager ( new ModemManager() ),
  mediaManager ( new MediaManager(this) ),
  m_lockedFD(lockedFD),
  volumeMountPoint ( QString() ),
  volumesCount ( 0 ),
  daemonSettings ( new QSettings ( "/opt/ubudsl/.daemon_ng.ini", QSettings::IniFormat ) ),
  m_sys ( UbuDSL::SystemInfo::getDistributionName() ),
  m_ver ( UbuDSL::SystemInfo::getDistributionRelease() ),
  m_canExit(true),
  m_endState(endState),
  m_dnsServers(dnsServers)
{
	if ( debug )
		fprintf( pFile, "UbuDSL_Thread::UbuDSL_Thread(): ctor called.\n" );
	
    this->login         = login;
    this->password      = password;
    this->protocol      = protocol;
    this->VPI           = VPI;
    this->VCI           = VCI;

    daemonSettings->setValue ( "protocol", ( protocol == 0 ? "PPPoA" : "PPPoE" ) );
    daemonSettings->setValue ( "VPI", VPI );
    daemonSettings->setValue ( "VCI", VCI );
    daemonSettings->setValue ( "service", service );
    daemonSettings->setValue ( "connect_at_boot", connectAtBoot );
    daemonSettings->setValue ( "connect_at_modem_plug", connectAtModemPlug );

    if ( !QFile::exists ( "configs/system.ini" ) )
        qWarning() << "configs/system.ini file not found!";
    
    if ( endState != NULL )
    	*endState = Success;

    systemIni = new QSettings ( "configs/system.ini", QSettings::IniFormat );

    systemIni->beginGroup ( m_sys + "_" + m_ver );

    packageManager = new PackageManager ( systemIni->value ( "package_manager", "" ).toString(), this );

    packagesList.clear();
    packagesToInstall.clear();
    cdIndex.clear();

    initDriversMap();
}


UbuDSL_Thread::~UbuDSL_Thread()
{
	if ( debug )
		fprintf( pFile, "UbuDSL_Thread::~UbuDSL_Thread(): dtor called.\n" );
    /*delete compiler;*/
    delete fileManager;
    delete packageManager;
    delete mediaManager;
    delete scriptManager;
}


void UbuDSL_Thread::run()
{
    emit message ( tr ( "Getting system informations..." ) );

    QString kernel   = UbuDSL::SystemInfo::getKernelRelease ();
    QString arch     = UbuDSL::SystemInfo::getArchitecture ();

    QRegExp kernelRegExp ( "([0-9]{1})\\.([0-9]{1})\\.([0-9]{1,2})" );
    int kernelNum = 0;

    if ( kernel.contains ( kernelRegExp ) )
    {
        QString str = kernelRegExp.cap ( 1 );
        str += kernelRegExp.cap ( 2 );
        str += kernelRegExp.cap ( 3 );

        kernelNum = str.toInt();
    }

    emit message ( QString ( tr ( "  Kernel: %1" ) ).arg ( kernel ) );
    emit message ( QString ( tr ( "  Architecture: %1" ) ).arg ( arch ) );
    emit message ( QString ( tr ( "  System: %1 %2" ) ).arg ( m_sys ).arg ( m_ver ) );

    emit detectingModem();
    emit message ( tr ( "Please plug your modem in" ) );

    modemManager->connectToDBus();

    connect ( modemManager, SIGNAL ( foundModem() ), this, SLOT ( quit() ) ); //after modem is found, quit from waiting loop
    exec();
    disconnect ( modemManager, SIGNAL ( foundModem() ), this, SLOT ( quit() ) );

    emit modemDetected ( modemManager->getModemName() );

	if ( debug )
		fprintf( pFile, "UbuDSL_Thread::run(): detected modem: %s.\n", qPrintable(modemManager->getModemName()) );
    
    daemonSettings->setValue ( "modem",      modemManager->getModemName() );
    daemonSettings->setValue ( "vendor_id",  modemManager->getVendorID() );
    daemonSettings->setValue ( "product_id", modemManager->getProductID() );
    daemonSettings->setValue ( "udi",        modemManager->getUDI() );
    driverName     = modemManager->getDriverName();
    driverVersion  = modemManager->getDriverVersion();
    m_driver = driverName + "-" + driverVersion;

    delete modemManager; //because it is no more needed

    if ( m_lockedFD > 0 )
    	unlockFile( m_lockedFD );
    
    QStringList notInstalledBasic ( packageManager->getNotInstalled ( systemIni->value ( "basic_from_cd", QStringList() ).toStringList() ) );

    if ( debug )
	fprintf( pFile, "UbuDSL_Thread::run(): not installed basic packages: %s\n", qPrintable(notInstalledBasic.join(", ")) );

    m_canExit = false;
	
    if ( !notInstalledBasic.isEmpty() && volumeMountPoint.isEmpty() )
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL_Thread::run(): waiting for disc (001).\n" );
    	
        waitForDisc();
    }

    if ( !packageManager->getArePackagesAvailable ( notInstalledBasic ) )
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL_Thread::run(): some basic packages are not available, adding CD.\n" );
    	
        packageManager->addCD ( volumeMountPoint );
    }

    if ( packageManager->installPackages ( notInstalledBasic ) == 0 )
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL_Thread::run(): basic packages installed successfully.\n" );
    	
        emit message ( tr ( "Basic packages from CD installed successfully" ) );
    }
    else
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL_Thread::run(): basic packages' installation failed.\n" );
    	
        emit message ( tr ( "Basic packages from CD installation failed!" ) );
    }

    if ( m_driver == "unicorn-neo" || ( m_driver == "ueagle-atm-1.4" && kernelNum < 2624 ) )
    {
    	if ( debug )
    		fprintf( pFile, "UbuDSL_Thread::run(): entering into the development actions if statement.\n" );
    	
        QStringList notInstalledDevel = packageManager->getNotInstalled( systemIni->value ( "devel_from_cd", QStringList() ).toStringList() );
        
    	if ( debug )
    		fprintf( pFile, "UbuDSL_Thread::run(): list of not installed development packages: %s.\n", qPrintable(notInstalledDevel.join(", ")) );
        
        if ( !notInstalledDevel.isEmpty() && volumeMountPoint.isEmpty() )
        {
        	if ( debug )
        		fprintf( pFile, "UbuDSL_Thread::run(): waiting for disc (002).\n" );
        	
            waitForDisc();
        }

        if ( !packageManager->getArePackagesAvailable ( notInstalledDevel ) )
        {
        	if ( debug )
        		fprintf( pFile, "UbuDSL_Thread::run(): some development packages are not available, adding CD.\n" );
        	
            packageManager->addCD ( volumeMountPoint );
        }

        emit message ( tr ( "Installing development packages..." ) );

        if ( packageManager->installPackages ( notInstalledDevel ) == 0 )
        {
        	if ( debug )
        		fprintf( pFile, "UbuDSL_Thread::run(): development packages installed successfully.\n" );
        	
            emit message ( tr ( "Development packages from CD installed successfully" ) );
        }
        else
        {
        	if ( debug )
        		fprintf( pFile, "UbuDSL_Thread::run(): development packages' installation failed.\n" );
        	
            emit message ( tr ( "Development packages from CD installation failed!" ) );
        }

        emit message ( tr ( "Preparing driver..." ) );

        Compiler::ModemDriver driver;
        QListIterator<Compiler::ModemDriver> it ( m_drivers );

        while ( it.hasNext() )
        {
            driver = it.next();
            
        	if ( debug )
        		fprintf( pFile, "UbuDSL_Thread::run(): iterating through the drivers, current is %s.\n", qPrintable(driver.name) );

            if ( driver.name == driverName && driver.version == driverVersion &&
                    arch == "amd64" && driver.amd64_compatible )
            {
            	if ( debug )
            		fprintf( pFile, "UbuDSL_Thread::run(): incompatible architecture!\n" );
            	
                emit incompatibleArch();
                
                if (m_endState != NULL)
                	*m_endState = IncompatibleArchitecture;
                
                return;
            }
            else if ( driver.name == driverName && driver.version == driverVersion )
            {
                Compiler *compiler = new Compiler ( driver, 0 );
                
            	if ( debug )
            		fprintf( pFile, "UbuDSL_Thread::run(): backuping driver.\n" );
            	
                compiler->backupDriver();
                
            	if ( debug )
            		fprintf( pFile, "UbuDSL_Thread::run(): extracting driver.\n" );
            	
                compiler->extractDriver();
                
            	if ( debug )
            		fprintf( pFile, "UbuDSL_Thread::run(): patching driver.\n" );
            	
                compiler->patchDriver();

                emit message ( tr ( "  Compiling driver..." ) );
                
            	if ( debug )
            		fprintf( pFile, "UbuDSL_Thread::run(): compiling driver.\n" );
            	
                if ( compiler->compileDriver() == 0 )
                {
                    emit message ( tr ( "  Installing driver" ) );
                    
                	if ( debug )
                		fprintf( pFile, "UbuDSL_Thread::run(): installing driver.\n" );
                    
                    if ( compiler->installDriver() != 0 )
                    {
                        emit message ( tr ( "    Driver installation failed!" ) );
                        emit message ( tr ( "    Restoring old driver..." ) );

                    	if ( debug )
                    		fprintf( pFile, "UbuDSL_Thread::run(): restoring driver.\n" );
                        
                        compiler->restoreDriver();
                        
                        if (m_endState != NULL)
                        	*m_endState = Failed;
                    }
                }
                else
                {
                    compiler->restoreDriver();
                    emit message ( tr ( "    Driver compilation failed!" ) );
                    
                    if (m_endState != NULL)
                    	*m_endState = Failed;
                }
                delete compiler;
                break;
            }
        }
    }

    emit message ( tr ( "Installing firmware..." ) );
    Firmware *fw = new Firmware;
    if ( !fw->installUeagleFirmware ( "firmware/ueagle-firmware.tar" ) )
        emit message ( tr ( "    Failed to install firmware: ueagle-atm!" ) );
    if ( !fw->installSpeedtchFirmware ( "firmware/speedtch-firmware.tar" ) )
        emit message ( tr ( "    Failed to install firmware: speedtch!" ) );
    if ( !fw->installCxacruFirmware ( "firmware/cxacru-firmware.tar" ) )
        emit message ( tr ( "    Failed to install firmware: cxacru!" ) );
    delete fw;

    m_canExit = true;
    
    emit message ( tr ( "Creating scripts..." ) );

    if ( !scriptManager->writeOptionsScript() )
        emit message ( tr ( "  Could not write ppp options script!" ) );
    if ( !scriptManager->writeSecretScripts ( login, password ) )
        emit message ( tr ( "  Could not write authorization scripts!" ) );
    if ( !scriptManager->writeUnicornScript ( login ) )
        emit message ( tr ( "  Could not write unicorn connection script!" ) );

    QString modprobeOptions(systemIni->value("modprobe_path", "/etc/modprobe.d/options").toString());
    
    if ( protocol == 0 )
    {
        if ( !scriptManager->writePPPoAScript ( VPI, VCI, login ) )
            emit message ( tr ( "  Could not write PPPoA connection script!" ) );
        
        scriptManager->writeUnicornOption(modprobeOptions, VPI, VCI, protocol);
    }
    else
    {
        if ( !scriptManager->writePPPoEScript ( login ) )
            emit message ( tr ( "  Could not write PPPoE connection script!" ) );
        
        scriptManager->writeUnicornOption(modprobeOptions, VPI, VCI, protocol);
    }

    scriptManager->setScriptsPermissions();

    DNS_Manager dns;

    if ( !dns.setServers(m_dnsServers) /*|| !dns.isFileCorrect()*/ )
    {
        emit message( tr("Setting OpenDSN as the default DNS servers...") );
	QStringList OpenDNS( QStringList() << "208.67.222.222" << "208.67.220.220" );
        dns.setServers ( OpenDNS  );
        daemonSettings->setValue( "DNS", OpenDNS );
    }
    else
    {
	daemonSettings->setValue( "DNS", m_dnsServers );
    }
    
    
    emit message( tr( "Loading the driver..." ) );
    
    if ( driverName == "unicorn" )
    	driverName = "unicorn_usb_eth";
    
    QProcess::execute( "modprobe", QStringList() << "-r" << "-f" << driverName );
    QProcess::execute( "modprobe", QStringList() << driverName );
    
    quit();
}


void UbuDSL_Thread::volumesListChanged()
{
    int count = mediaManager->getVolumesList().count();
    if ( count != volumesCount )
    {
        volumesCount = count;

        volumes = mediaManager->getVolumesList();
        emit setVolumesList ( volumes );
    }
}


void UbuDSL_Thread::volumeSelected ( const int &index )
{
    if ( index == -1 ) //ex. called from gui
    {
        qDebug() << "UbuDSL_Thread::volumeSelected(): index == -1, skipping.";
        emit discMounted();
    }

    if ( index > volumes.count() - 1 || index < 0 )
    {
    	qDebug() << "UbuDSL_Thread::volumeSelected(): index > volumes.count() - 1 || index < 0. (volumes.count:" << volumes.count() <<  ").";
        return;
    }

    if ( mediaManager->getVolumeIsMounted ( volumes[index].getUdi() ) )
    {
        qDebug() << "UbuDSL_Thread::volumeSelected(): volume is mounted.";
        volumeMountPoint = mediaManager->getVolumeMountPoint ( volumes[index].getUdi() );
    }
    else
    {
        qDebug() << "UbuDSL_Thread::volumeSelected(): volume is NOT mounted.";
        QString mntPoint = "/tmp/ubudsl_mnt";
        int mountingResult = mediaManager->mountVolume ( volumes[index].getUdi(), mntPoint );

        if ( mountingResult == 0 )
        {
            qDebug() << "UbuDSL_Thread::volumeSelected(): volume mounted in" << mntPoint;
            volumeMountPoint = mntPoint;
        }
        else
        {
            qDebug() << "UbuDSL_Thread::volumeSelected(): could NOT mount volume in" << mntPoint;
            volumeMountPoint = QString();
        }
    }

    emit discMounted();
}


void UbuDSL_Thread::waitForDisc()
{
    emit message ( tr ( "Waiting for disc..." ) );
    emit setVolumesList ( mediaManager->getVolumesList() );
    emit waitingForDisc();
    volumes = mediaManager->getVolumesList();
    connect ( mediaManager, SIGNAL ( volumeAdded() ), this, SLOT ( volumesListChanged() ) );
    connect ( mediaManager, SIGNAL ( volumeRemoved() ), this, SLOT ( volumesListChanged() ) );
    connect ( this, SIGNAL ( discMounted() ), this, SLOT ( quit() ) );
    exec();
    disconnect ( this, SIGNAL ( discMounted() ), this, SLOT ( quit() ) );
    disconnect ( mediaManager, SIGNAL ( volumeAdded() ), this, SIGNAL ( volumesListChanged() ) );
    disconnect ( mediaManager, SIGNAL ( volumeRemoved() ), this, SIGNAL ( volumesListChanged() ) );
}

void UbuDSL_Thread::initDriversMap()
{
    Compiler::ModemDriver sagem_e4;
    sagem_e4.name = "ueagle-atm";
    sagem_e4.version = "1.4";
    sagem_e4.extracted_driver_path = "/tmp/ueagle-atm-1.4";
    sagem_e4.compiled_driver_name = "ueagle-atm.ko";
    sagem_e4.extraction_path = "/tmp";
    sagem_e4.driver_bundle_path = "drivers/ueagle-atm-1.4.tar";
    sagem_e4.compilation_arguments = "all";
    sagem_e4.installation_arguments = "install";
    sagem_e4.environment_variables = "PWD=/tmp/ueagle-atm-1.4, LC_ALL=C";
    sagem_e4.amd64_compatible = true;

    Compiler::ModemDriver zxdsl;
    zxdsl.name = "unicorn";
    zxdsl.version = "neo";
    zxdsl.extracted_driver_path = "/tmp/unicorn-neo";
    zxdsl.compiled_driver_name = "unicorn_usb_eth.ko";
    zxdsl.extraction_path = "/tmp";
    zxdsl.driver_bundle_path = "drivers/unicorn-neo.tar";
    zxdsl.compilation_arguments = "clean, modules";
    zxdsl.installation_arguments = "modules_install";
    zxdsl.environment_variables = "PWD=/tmp/unicorn-neo, LC_ALL=C";
    zxdsl.amd64_compatible = false;

    m_drivers.append ( sagem_e4 );
    m_drivers.append ( zxdsl );
}

bool UbuDSL_Thread::getCanExit()
{
	return m_canExit;
}
