/*
 * File: VolumeSelector.cpp
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU GPL v3 or newer
 */

#include "VolumeSelector.h"
EjectThread::EjectThread()
: m_volume( "" ),
  m_mediaManager( new MediaManager(this) )
{
}

EjectThread::~EjectThread()
{
	delete m_mediaManager;
}

void EjectThread::run()
{
    if ( m_volume.isEmpty() )
        return;

    m_mediaManager->ejectVolume ( m_volume );

    emit volumeEjected ( m_volume );
}

VolumeSelector::VolumeSelector ( QWidget *parent )
        : QDialog ( parent )
{
    ui.setupUi ( this );
    ui.buttonOK->setEnabled ( false );
    ui.buttonEject->setEnabled ( false );
    ui.volumesList->setSortingEnabled ( true );

    setAttribute ( Qt::WA_QuitOnClose, false );

    eject        = new EjectThread;
    mediaManager = new MediaManager(eject);

    m_quit = false;

    connect ( eject          , SIGNAL ( volumeEjected ( const QString & ) ),
              this           , SLOT ( removeFromEjectingList ( const QString & ) ) );
    connect ( ui.volumesList , SIGNAL ( currentItemChanged ( QListWidgetItem *, QListWidgetItem * ) ),
              this           , SLOT ( updateButtons() ) );
    connect ( ui.buttonOK    , SIGNAL ( clicked() ),
              this           , SLOT ( volumeSelected() ) );
    connect ( ui.buttonEject , SIGNAL ( clicked() ),
              this           , SLOT ( ejectVolume() ) );
    connect ( ui.buttonCancel, SIGNAL ( clicked() ),
              this           , SLOT ( cancel() ) );
    connect ( mediaManager   , SIGNAL ( volumeAdded ( const QString &, const QString &, const QString & ) ),
              this           , SLOT ( addVolume ( const QString &, QString, const QString & ) ) );
    connect ( mediaManager   , SIGNAL ( volumeRemoved ( const QString & ) ),
              this           , SLOT ( removeVolume ( const QString & ) ) );

    QList<Volume> volumes = mediaManager->getVolumesList();
    for ( int i = 0; i < volumes.count(); i++ )
        addVolume ( volumes[i].getDevice(), volumes[i].getLabel(), volumes[i].getUdi() );
}

VolumeSelector::~VolumeSelector()
{

}

void VolumeSelector::closeEvent ( QCloseEvent *e )
{
    if ( m_quit )
    {
        e->accept();
    }
    else
    {
        emit cancel();
        e->ignore();
    }
}

void VolumeSelector::addVolume ( const QString &blockDev, QString label, const QString &udi )
{
    if ( label.isEmpty() )
        label = tr ( "Untitled", "For translators: \"Untitled\" - volume." );

    QString volumeStr       = blockDev + ": " + label;
    QListWidgetItem *volume = new QListWidgetItem ( QIcon ( ":images/media-optical.png" ), volumeStr );

    volume->setData ( Qt::UserRole, udi );

    QList<QListWidgetItem *> items = ui.volumesList->findItems ( "*", Qt::MatchWildcard );
    QListIterator<QListWidgetItem *> it ( items );

    while ( it.hasNext() )
    {
        qDebug() << 23;
        QListWidgetItem *cItem = it.next();
        if ( cItem->data ( Qt::UserRole ).toString() == udi )
            return;
    }

    ui.volumesList->addItem ( volume );
}

void VolumeSelector::removeVolume ( const QString &udi )
{
    QList<QListWidgetItem *> items = ui.volumesList->findItems ( "*", Qt::MatchWildcard );
    QListIterator<QListWidgetItem *> it ( items );

    while ( it.hasNext() )
    {
        QListWidgetItem *cItem = it.next();

        if ( cItem->data ( Qt::UserRole ) == udi )
        {
            int row = ui.volumesList->row ( cItem );
            ui.volumesList->takeItem ( row );

            delete cItem;
            return;
        }
    }
}

void VolumeSelector::updateButtons()
{
    if ( ui.volumesList->currentItem() == 0 ||
            volumesBeingEjected.contains ( ui.volumesList->currentItem()->data ( Qt::UserRole ).toString() ) )
    {
        ui.buttonOK->setEnabled ( false );
        ui.buttonEject->setEnabled ( false );
    }
    else
    {
        ui.buttonOK->setEnabled ( true );
        ui.buttonEject->setEnabled ( true );
    }
}

void VolumeSelector::volumeSelected()
{
    if ( ui.volumesList->currentItem() != 0 )
    {
        accept();
        qDebug() << "Volume selected:" << ui.volumesList->currentItem()->text();
        QStringList volumes;
        foreach ( Volume volume, mediaManager->getVolumesList() )
        {
        	volumes.append( volume.getUdi() );
        }
        const int index = volumes.indexOf ( ui.volumesList->currentItem()->data ( Qt::UserRole ).toString() );

        if ( index > -1 )
        {
            qDebug() << "VolumeSelector::volumeSelected(): index:" << index;
            emit volumeSelected ( index );
        }
        else
            qDebug() << "VolumeSelector::volumeSelected(): NOT ok";
    }
}

void VolumeSelector::ejectVolume()
{
    if ( ui.volumesList->currentItem() != 0 )
    {
        qDebug() << "Ejecting volume:" << ui.volumesList->currentItem()->text();

        ui.volumesList->currentItem()->setIcon ( QIcon ( ":images/media-ejecting.png" ) );
        volumesBeingEjected.append ( ui.volumesList->currentItem()->data ( Qt::UserRole ).toString() );

        eject->setVolume ( ui.volumesList->currentItem()->data ( Qt::UserRole ).toString() );
        eject->start();

        updateButtons();
    }
}

void VolumeSelector::removeFromEjectingList ( const QString &volume )
{
    volumesBeingEjected.removeAll ( volume );

    QList<QListWidgetItem *> items = ui.volumesList->findItems ( "*", Qt::MatchWildcard );
    QListIterator<QListWidgetItem *> it ( items );

    while ( it.hasNext() )
    {
        QListWidgetItem *cItem = it.next();

        if ( cItem->data ( Qt::UserRole ) == volume )
        {
            cItem->setIcon ( QIcon ( ":images/media-optical.png" ) );
            updateButtons();
            return;
        }
    }
}

void VolumeSelector::cancel()
{
    QMessageBox *msgBox = new QMessageBox ( this );
    msgBox->setWindowTitle ( tr ( "Cancel dialog" ) );
    msgBox->setText ( tr ( "What do you want to do?" ) );
    msgBox->setAttribute ( Qt::WA_QuitOnClose, false );

    QAbstractButton *buttonQuit     = msgBox->addButton ( tr ( "Abort configuration" ),     QMessageBox::ActionRole );
    QAbstractButton *buttonContinue = msgBox->addButton ( tr ( "Back to Volume Selector" ), QMessageBox::AcceptRole );
    QAbstractButton *buttonSkip     = msgBox->addButton ( tr ( "Continue without disc" ),   QMessageBox::ActionRole );

    Q_UNUSED ( buttonContinue );

    msgBox->exec();

    if ( msgBox->clickedButton() == buttonQuit ) //TODO: do right actions
        emit quit();
    else if ( msgBox->clickedButton() == buttonSkip )
    {
        qDebug() << "skip";
        m_quit = true;
        this->close();
        emit volumeSelected ( -1 );
    }
}
