/*
 * File: main.cpp
 * Created: Mon 31 Mar 2008
 * Last modified: Tue 23 Apr 2008
 * Author: Adrian Łubik
 * License: GNU LGPL v3 or newer
 */

#ifdef BUILDING_PKG
#include "../libubudsl/FileLocker.h"
#include "../libubudsl/SystemInfo.h"
#include "../libubudsl/Globals.h"
#else
#include "libubudsl/FileLocker.h"
#include "libubudsl/SystemInfo.h"
#include "libubudsl/Globals.h"
#endif

#include "CustomStyle.h"
#include "UbuDSL_GUI.h"
#include "UbuDSL_Cmd.h"
#include <unistd.h>

#include <QApplication>
#include <QCoreApplication>
#include <QTranslator>
#include <QObject>
#include <QMutex>
#include <QFile>
#include <QString>

extern bool  debug;
extern bool  debugToFile;
extern FILE *pFile;

QMutex mutex;

void debugHandler ( QtMsgType type, const char *msg );
void loadTranslation();
void showHelp();
bool checkCompatibility ( QString &sys, QString &ver );
bool getIsRoot();


int main ( int argc, char *argv[] )
{
    QFile::remove ( "/opt/ubudsl/debug.txt" );
    //qInstallMsgHandler ( debugHandler );

    setDebug( true/*, "/opt/ubudsl/debug.txt"*/ );
    bool cmdLine = false;
    bool openDNS = false;
    int  appExec = 0;

    for ( int i = 0; i < argc; i++ )
    {
    	QString arg( argv[i] );
        if ( arg == "-c" || arg == "--command-line" )
        {
            cmdLine = true;
            break;
        }
        else if ( arg == "--set-opendns" )
        {
        	setDebug( false );
        	openDNS = true;
        	cmdLine = true;
        }
        else if ( arg == "-h" || arg == "--help" )
        {
        	showHelp();
        	return 0;
        }
    }

    //Needed for check whether dpkg file is locked
    int FD = 0;
    if ( QFile::exists ( "/var/lib/dpkg/lock" ) )
        FD = lockFile ( "/var/lib/dpkg/lock" );

//    QTranslator appTranslator;

    if ( cmdLine )
    {
        QCoreApplication app ( argc, argv );

        if ( !QDir::setCurrent ( app.applicationDirPath() ) )
            qDebug() << "Could not set current directory as:" << app.applicationDirPath() + "/../";
        
        loadTranslation();
        
        if ( openDNS )
        {
        	if ( !getIsRoot() )
        		return -1;
        	
        	QStringList servers;
        	servers << "208.67.222.222" << "208.67.220.220";
        	
        	DNS_Manager dns;
        	
        	if ( dns.setServers( servers ) )
        	{
        		QString msg = QObject::tr("The following DNS servers have ben successfully set:\n%1");
        		printf( "%s\n", qPrintable(msg.arg(servers.join("\n"))) );
        		return 0;
        	}
        	else
        	{
        		QString msg = QObject::tr("An error occured while setting the OpenDNS servers!");
        		printf( "%s\n", qPrintable(msg) );
        		return -2;
        	}
        }

        if ( FD < 0 )
        {
            QString msg = QObject::tr ( "A working package manager was detected (apt-get or synaptic, or ...).\n"
                                        "UbuDSL can not work while package manager is running up. Please close "
                                        "it\n and then run UbuDSL again." );
            fprintf ( stderr, "%s\n", qPrintable ( msg ) );
            return -2;
        }

        QString sys, ver;

        if ( !checkCompatibility ( sys, ver ) )
        {
            QString msg = QObject::tr ( "Your system (%1 %2) is unsupported." );
            fprintf ( stderr, "%s\n", qPrintable ( msg.arg ( sys, ver ) ) );
            qFatal ( qPrintable ( QString ( "Unsupported system (%1 %2)" ).arg ( sys, ver ) ) );
        }
        
        if ( !getIsRoot() )
        	return -1;

        UbuDSL_Cmd *cmd = new UbuDSL_Cmd( FD );
        QObject::connect ( cmd, SIGNAL ( finished() ), &app, SLOT ( quit() ) );

        appExec = app.exec();
        
        delete cmd;
    }
    else
    {
        QApplication app ( argc, argv );
        app.setStyle(new CustomStyle());

	QSettings palette( ":palette.ini", QSettings::IniFormat );
	app.setPalette( palette.value("palette").value<QPalette>() );

        if ( !QDir::setCurrent ( app.applicationDirPath() ) )
            qDebug() << "Could not set current directory as:" << app.applicationDirPath() + "/../";
        
        loadTranslation();

        QString sys, ver;

        if ( !checkCompatibility ( sys, ver ) )
        {
            QString title = QObject::tr ( "Unsupported system" );
            QString text  = QObject::tr ( "Your system (%1 %2) is currently unsupported.\n\n"
                                          "Application will now quit." );
            QMessageBox::critical ( 0, title, text.arg ( sys, ver ) );

            qFatal ( qPrintable ( QString ( "Unsupported system (%1 %2)" ).arg ( sys, ver ) ) );
        }

        QString title = QObject::tr ( "Critical error" );

        if ( !getIsRoot() )
        {
            QString text = QObject::tr ( "UbuDSL has not been run with the administrator's "
                                         "privileges. Application will now quit." );
            QMessageBox::critical ( 0, title, text );

            return -1;
        }

        if ( FD < 0 )
        {
            QString text = QObject::tr ( "UbuDSL has detected a running package manager "
                                         "like apt-get or synaptic. Application can not "
                                         "work while it is running up.\n\n"
                                         "Please close all the package managers and then "
                                         "run UbuDSL again." );
            QMessageBox::critical ( 0, title, text );

            return -2;
        }


        QDesktopWidget *desktop = app.desktop();
        UbuDSL_GUI *gui = new UbuDSL_GUI ( FD, desktop );

        app.connect ( gui, SIGNAL ( quit() ), &app, SLOT ( quit() ) );

        gui->move ( ( desktop->width() - gui->width() ) / 2, ( desktop->height() - gui->height() ) / 2 );
        gui->show();

        appExec = app.exec();
        
        delete gui;
        delete desktop;
    }

    return appExec;
}

bool checkCompatibility ( QString &sys, QString &ver )
{
    QString cfgFile = QCoreApplication::instance()->applicationDirPath();

    if (QFile::exists( cfgFile + "/configs/system.ini" ))
        cfgFile += "/configs/system.ini";
    else if (QFile::exists( cfgFile + "/../configs/system.ini" ))
        cfgFile += "/../configs/system.ini";
    else
        cfgFile  = ":configs/system.ini";

    QSettings s ( cfgFile, QSettings::IniFormat );
    QStringList groups = s.childGroups();

    sys = UbuDSL::SystemInfo::getDistributionName();
    ver = UbuDSL::SystemInfo::getDistributionRelease();

    return groups.contains ( sys + "_" + ver );
}

void debugHandler ( QtMsgType type, const char *msg )
{
    mutex.lock();

    fprintf( pFile, "%s\n", msg );

    if ( type == QtFatalMsg && debugToFile )
    {
    	fprintf( pFile,  "Fatal: %s\n", msg );
        fprintf( stderr, "Fatal: %s\n", msg );
        
        abort();
    }
    else if ( type == QtFatalMsg && !debugToFile )
    {
    	fprintf( pFile, "Fatal: %s\n", msg );
    	
    	abort();
    }
    else
    {
    	fprintf( pFile, "%s\n", msg );
    }

    mutex.unlock();
}

void loadTranslation()
{
    QString translatorFileName = QLatin1String("ubudsl_");
    translatorFileName += QLocale::system().name();

    QTranslator *translator = new QTranslator(QCoreApplication::instance());

    if (!translator->load(translatorFileName, QString(QCoreApplication::instance()->applicationDirPath() + "/translations/")))
    {
        qDebug() << "Failed to load the" << QString(QCoreApplication::instance()->applicationDirPath() + "/translations/" + translatorFileName + "!");

        if (!translator->load(translatorFileName, QString(QCoreApplication::instance()->applicationDirPath() + "/../translations/")))
        {
            qDebug() << "Failed to load the" << QString(QCoreApplication::instance()->applicationDirPath() + "/../translations/" + translatorFileName + "!");

            if (!translator->load(QString(":translations/" + translatorFileName)))
            {
                qDebug() << "Failed to load the" << QString(":translations/" + translatorFileName + "!");
            }
        }
    }

    QCoreApplication::instance()->installTranslator(translator);
}

void showHelp()
{
	QString help = QObject::tr( "UbuDSL - an ADSL connection configuration tool.\n"
		                    "You can run UbuDSL with the following arguments:\n\n"
		                    "    -h or --help:           Show this help message\n"
		                    "    -c or --command-line:   Run in text mode.\n"
                                    "          --set-opendns:    Set OpenDNS servers." );
	
	printf( "%s\n", qPrintable( help ) );
}

bool getIsRoot()
{
	int UID = getuid();
	
    if ( UID != 0 )
    {
        QString msg = QObject::tr ( "You are not the administrator!\n"
                                    "UbuDSL must be run with the administrator's privileges!" );
        fprintf ( stderr, "%s\n", qPrintable ( msg ) );
        return false;
    }
    else
    	return true;
}

