#ifndef DAEMON_H_
#define DAEMON_H_

#include <QtCore>
#include <QtDBus>
#include <QtNetwork>

#include <sys/klog.h>
#include <signal.h>

#include "UbuDSL_Message.h"
#include "DaemonAdaptor.h"
#include "ConnectThread.h"
#include "RestartModem.h"
#include "CompilerThread.h"
#include "Global.h"
#include "libubudsl/libubudsl.h"

//extern bool debug;

class UbuDSL_Daemon : QObject
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.ubudsl.Daemon")
 
public:
	UbuDSL_Daemon(QObject* parent = 0);
	virtual ~UbuDSL_Daemon();
    QDBusConnection dbus;
 
    static int        getModemState();
    static void       debugHandler( QtMsgType type, const char *msg );
public slots:
//const:
	const QString getIP(QString interface = "");
	const QString getDefaultRoute();

//void:
	void    setConnectAtBoot(bool connect);
	void    setConnectAtModemPlug(bool connect);
	void    establishConnection();
	void    terminateConnection( bool message = true );
	void    restartModem();
	void    setIdlenessTime(const int &idleness);
	void    useNewModem();
	void    setAccountData( const QString &login, const QString &password );

//bool:
	bool    getIsModemPlugged();
	bool    getIsCompiling();
	bool    getConnectAtBoot()      { return m_settings->value("connect_at_boot", false).toBool(); }
	bool    getConnectAtModemPlug() { return m_settings->value("connect_at_modem_plug", false).toBool(); };
	bool    reply()                 { return true; } //called by applet to check connection with daemon
	
//QString:
	QString getVPI()                { return m_settings->value("vpi", "error").toString(); }
	QString getVCI()                { return m_settings->value("vci", "error").toString(); }
	QString getProtocol()           { return m_settings->value("protocol", "error").toString(); }
	QString getService()            { return m_settings->value("service", "error").toString(); }
	QString getInterface();

	int getConnectionState();
	int getIdlenessTime(); //From settings
	
	QStringList getLoginAndPassword();

private slots:
	void connectingStarted();
	void connectingFinished();
	static void writeLog(const QString &str);
	void deviceAdded( const QString udi );
	void deviceRemoved( const QString udi );
	void verifyDevice();
	void verifyDNS();
	void getTransmissionStatistics(QString interface = "");
	QTime getConnectionTime();
	void debugConnecting(QByteArray msg);
	void compilationFinished();
	void compilationStarted();
	
private:
	void writeConnectionLog(const QString &str);
	bool readUnsignedFromFile( const QString &fileName, unsigned int &value );
	bool readLongFromFile( const QString &fileName, unsigned  long &value );
	void initModemsMap();
	void setOpenDNS();
	void removePidFiles( const QString &killedIface = "ppp0" );
	
	typedef struct Modem
	{
		QString name;
		QString driver;
		QMultiMap<int, int> usb_id;
	};
	
	bool m_justRun;
	bool m_connecting;
	bool m_modemIsPlugged;
	bool m_compiling;
	
	ConnectThread *m_connectThread;
	
	int m_newVendorID;
	int m_newProductID;
	
	QDBusInterface *m_halManager;
//	QDBusInterface *m_halDevice;
	
	static QFile *m_logFile;
	
	QList<Modem> m_modems;
	
	static QMutex m_mutexLog;
	QMutex m_connectMutex;
	QMutex m_disconnectMutex;
	
	QRegExp m_connectedRegExp;
	QRegExp m_connectingRegExp;
	QRegExp m_invalidDataRegExp;
	
	static QSettings *m_settings;
	
	QString m_IP;
	QString m_lastUDI;
	QString m_newModemName;
	QString m_newUDI;
	
	QTime   m_connectionTime;
	
	QTimer *m_deviceTimer;
	QTimer *m_timer;
	
	unsigned long m_rx_bytes;
	unsigned long m_tx_bytes;
	
	DNS_Manager     dns;
	RestartModem   *m_restartModem;
	CompilerThread *m_compilerThread;
	
	
signals:
	Q_SCRIPTABLE void transmissionSpeed( const unsigned int &rx, const unsigned int &tx );
	Q_SCRIPTABLE void transmissionErrors( const unsigned int &rx, const unsigned int &tx );
	Q_SCRIPTABLE void transmissionBytes( const double &rx, const double &tx );
	Q_SCRIPTABLE void IP( const QString &ip );
	Q_SCRIPTABLE void defaultRoute( const QString &iface );
	Q_SCRIPTABLE void connectionTime( const int &h, const int &m, const int &s );

	Q_SCRIPTABLE void modemPlugged( const QString &name );
	Q_SCRIPTABLE void newModemPlugged( const QString &name );
	Q_SCRIPTABLE void modemUnplugged();
	Q_SCRIPTABLE void newModemUnplugged();
	Q_SCRIPTABLE void incorrectDNS();
	Q_SCRIPTABLE void settingOpenDNS( const bool success );
	
	Q_SCRIPTABLE void connectionEstablished();
	Q_SCRIPTABLE void connectionTerminated();
	Q_SCRIPTABLE void establishingConnection();
	Q_SCRIPTABLE void setEnabledConnect(bool enable);
	Q_SCRIPTABLE void connectionState(const int &state);
	Q_SCRIPTABLE void connectionTimeout();
	Q_SCRIPTABLE void restartModemStarted();
	Q_SCRIPTABLE void restartModemFinished();
	Q_SCRIPTABLE void accountDataChanged( const bool success );
	
	Q_SCRIPTABLE void compilingDriver();
	Q_SCRIPTABLE void compilationFinished( const bool success );
	Q_SCRIPTABLE void synchronizing();
	Q_SCRIPTABLE void synchronizationFailed();
	Q_SCRIPTABLE void invalidAccountData();
};

#endif /*DAEMON_H_*/
